//LPS331 I2C Version

#define LPS331_i2c_addr 0xBA      //address: b110100xx  (2nd bit = 0 with ADO = gnd; 2nd bit = 1 with AD0=VCC)
                                   //1st bit is R/W with R==1 and W==0

#define LPS331_W_ADDRESS   LPS331_i2c_addr             //  ( BIT0 ==0 for writes)
#define LPS331_R_ADDRESS   (LPS331_i2c_addr | 0x01)    //  (BIT0 ==1 for reads)


/*  NOTES: Temp working; Pressure appears to be working ...
*/

extern unsigned short v=0;

unsigned short addr;

float fVal = 0.0;




//device comes up in sleep mode on power-up, so need to set config reg to wake it up
//1 STEP:

// 1) SET SLEEP TO OFF   (REG: 0x20)   & output data rate

void init_LPS331()
{
    unsigned short address, dat;

// 1a) SET Output Data Rate to 1Hz
// 1b) SET SLEEP TO OFF   (REG: 0x20)
   address = 0x20;
   dat = 0x90;     //set sleep off (to wakeup) & set internal clock to gyro X axis (suggested by data-sheet)

   //LPS331_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(LPS331_W_ADDRESS, address, dat);


   delay_ms(1);

}


char get_LPS331_who_am_i()
{
   unsigned short address, dat;
   //register of who_am_i: 0x0F
   address = 0x0F;
   dat = 0;

   //v = LPS331_i2c_ReadReg(address, &dat);
	v = i2c1_Get_Data(LPS331_W_ADDRESS, address);

//should return dat = 0xBB = 187d
}

//reads a 2 byte channel
void read_LPS331_chan2(unsigned short address, unsigned long *sensorData)
{

   unsigned short dat;

   addr = address;

   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(LPS331_W_ADDRESS, addr);
   *sensorData = (unsigned long)v;

   addr = address + 1;  //low byte accelX
   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(LPS331_W_ADDRESS, addr);
   *sensorData += ((unsigned long)v<<8);


}

//reads a 3 byte channel
void read_LPS331_chan3(unsigned short address, unsigned long *sensorData)
{

   unsigned short dat;

   addr = address;

   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(LPS331_W_ADDRESS, addr);
   *sensorData = (unsigned long)v;    //read lowest byte

   addr = address + 1;  //middle byte
   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(LPS331_W_ADDRESS, addr);
   *sensorData += ((unsigned long)v<<8);

   addr= address + 2;  //high byte
   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(LPS331_W_ADDRESS, addr);
   *sensorData += ((unsigned long)v<<16);


}


//returns pressure in mBar SeaLevel
int convert_pres_to_seaLevel(int pressure_mBar, int elev_ft, int temp_degC)
{
    unsigned int seaLevelPressure;
    float slp_mBar, ratio, temp_degK, elev_m;

    elev_m = 12.0*elev_ft/39.4;

    temp_degK = 273.15 + (float)temp_degC;

    ratio = expf(-0.03416*elev_m/temp_degK);

    slp_mBar = (float)(pressure_mBar)/ratio;

    //slp_inHg = 29.5299875*slp_mBar;

    return((int)(slp_mBar+0.5));  //add 0.5 to provide rounding upon conversion back to int

}

//reads 3 byte pressure value
//returns val in mBars (note: 
void get_LPS331_pressure(int *pressure)
{
   unsigned long sens_data = 0;


    addr = 0x28;  //pressure starting register address
    read_LPS331_chan3(addr, &sens_data);
    *pressure = (int)(sens_data / (unsigned long)4096);  //sens_data/4096 => mBars

}

//reads 2 byte temperature value
//returns Temp in Deg C (rounded to nearest whole deg)
//note: this doesn't work; gives wrong answer; suspect datasheet screwed up
void get_LPS331_temperature(int *temperature)
{
   unsigned int sens_data = 0;


    addr = 0x2B;  //temperature starting register addr
    read_LPS331_chan2(addr, &sens_data);

    fVal =  (float)(((int)sens_data)/480) + 42.5;  //degC


    *temperature = (int)(fVal + 0.5);


}
