//MCP3551/3553 DRIVER.C
#include <plib.h>

int mcp3553_state = 0;
int mcp3553_busy = 0;	//busy if ==1

void init_mcp3553()
{
    unsigned int val, data, configBits;

    configBits = ANSELB;
    ANSELB = ANSELB & ~BIT_12 & ~BIT_15;
    configBits = ANSELA;

    mPORTBSetBits(BIT_12);  //!CS1
    mPORTBSetBits(BIT_15);  //!CS2

    PORTSetPinsDigitalOut(IOPORT_B, BIT_12);
    PORTSetPinsDigitalOut(IOPORT_B, BIT_15);


}


//pull cs line low to start conversion
void start_mcp3553(int chan)
{
	mcp3553_busy = 1;
    if(chan == 1)
    {
        mPORTBClearBits(BIT_10);
    }
    else if(chan == 2)
    {
        mPORTBClearBits(BIT_15);
    }
    else if(chan == 6)
    {
        mPORTBClearBits(BIT_6);
    }
    else if(chan == 7)
    {
        mPORTBClearBits(BIT_7);
    }


	//bring cs line low
	//mPORTBClearBits(BIT_15);		//!CS0 mcp3551
	delay_us(10);	//give it a chance to bring it's line from high-Z to driven high

	//start ISR on T2 to check for !dataRdy line low
//	clearT2InterruptFlag();
//	T2InterruptEnable = 1;
}

//pull cs line high to stop conversion and put chip into standby state
void stop_mcp3553(int chan)
{
    if(chan == 1)
    {
        mPORTBSetBits(BIT_10);
    }
    else if(chan == 2)
    {
        mPORTBSetBits(BIT_15);
    }
    else if(chan == 6)
    {
        mPORTBSetBits(BIT_6);
    }
    else if(chan == 7)
    {
        mPORTBSetBits(BIT_7);
    }

    //mPORTBSetBits(BIT_15);  //!CS0
//	T2InterruptEnable = 0;		//turn off timer2

	mcp3553_busy = 0;
	mcp3553_state = 0;
}

//returns 1 if not ready, return 0 if ready
//check spi1 input to see if mcp3553 has pulled it low (!DataRdy)
unsigned int spi1_check_mcp3553()
{
	unsigned int not_dataReady;

        //pcDuino bd
	not_dataReady =  PORTReadBits(IOPORT_B, BIT_11);  //read SDI/!DATA_READY (the mcp3551's SDO line; our SDI!

        //Deutsch HV precAn bd
        //not_dataReady =  PORTReadBits(IOPORT_B, BIT_13);  //read SDI/!DATA_READY (the mcp3551's SDO line; our SDI!

	return(not_dataReady&0x0000FFFF);
}

unsigned int spi2_check_mcp3553()
{
	unsigned int not_dataReady;

        //pcDuino bd
	//not_dataReady =  PORTReadBits(IOPORT_B, BIT_11);  //read SDI/!DATA_READY (the mcp3551's SDO line; our SDI!

        //Deutsch HV precAn bd
        not_dataReady =  PORTReadBits(IOPORT_B, BIT_13);  //read SDI/!DATA_READY (the mcp3551's SDO line; our SDI!

	return(not_dataReady&0x0000FFFF);
}

long spi1_process_mcp3553()
{
        //write to SPI buffer triggers and auto Tx's and Rx's byte
	SPI1BUF = 0x03;
	while(!SPI1STATbits.SPIRBF);

	unsigned int mask = (~BIT_7 & 0xFF) & (~BIT_6 & 0xFF) & (~BIT_5 & 0xFF);
	unsigned int highByteCh0 = (SPI1BUF)&mask;
	delay_us(3);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


	SPI1BUF = 0x02;
	while(!SPI1STATbits.SPIRBF);

	unsigned int midByteCh0 = (SPI1BUF );
	delay_us(10);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


	SPI1BUF = (0x01);
	while(!SPI1STATbits.SPIRBF);

	unsigned int lowByteCh0 = SPI1BUF ;
	delay_us(3);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


//bring !CS High (idle state)
	//mPORTBSetBits(BIT_12);		//!CS  mcp3901
//	mPORTBSetBits(BIT_15);		//!CS0 mcp3551


	unsigned int val = (highByteCh0<<8) + midByteCh0;
	unsigned int val2 = (midByteCh0<<8) + lowByteCh0;

//midByteCh0 = 0xFFFF;
//lowByteCh0 = 0xFFFF;

	unsigned int negative = BIT_4 & highByteCh0;

	long lval;

	if(negative > 0)
	{


		lval = ((highByteCh0&0xFFFFFFFF)<<16) + ((midByteCh0&0x0000FFFF)<<8) + lowByteCh0;

		lval = -(2097153 - lval);

	}
	else
	{
		lval = (highByteCh0<<16) + (midByteCh0<<8) + lowByteCh0;
	}



	return(lval);

}

long spi2_process_mcp3553()
{
    unsigned int v = 0;

        //write to SPI buffer triggers and auto Tx's and Rx's byte
	SPI2BUF = 0x03;

        do
        {
            v = SPI2STATbits.SPIRBF;
        }
	while(!SPI2STATbits.SPIRBF);

	unsigned int mask = (~BIT_7 & 0xFF) & (~BIT_6 & 0xFF) & (~BIT_5 & 0xFF);
	unsigned int highByteCh0 = (SPI2BUF)&mask;
	delay_us(3);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


	SPI2BUF = 0x02;
	while(!SPI2STATbits.SPIRBF);

	unsigned int midByteCh0 = (SPI2BUF );
	delay_us(10);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


	SPI2BUF = (0x01);
	while(!SPI2STATbits.SPIRBF);

	unsigned int lowByteCh0 = SPI2BUF ;
	delay_us(3);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


//bring !CS High (idle state)
	//mPORTBSetBits(BIT_12);		//!CS  mcp3901
//	mPORTBSetBits(BIT_15);		//!CS0 mcp3551


	unsigned int val = (highByteCh0<<8) + midByteCh0;
	unsigned int val2 = (midByteCh0<<8) + lowByteCh0;

//midByteCh0 = 0xFFFF;
//lowByteCh0 = 0xFFFF;

	unsigned int negative = BIT_4 & highByteCh0;

	long lval;

	if(negative > 0)
	{


		lval = ((highByteCh0&0xFFFFFFFF)<<16) + ((midByteCh0&0x0000FFFF)<<8) + lowByteCh0;

		lval = -(2097153 - lval);

	}
	else
	{
		lval = (highByteCh0<<16) + (midByteCh0<<8) + lowByteCh0;
	}



	return(lval);

}

//assumes cs lines have already been setup in spi.h (i.e. address lines)
//so no need, can just call stop_mcp3553 to bring cs line high
void old_init_mcp3553(unsigned int address, unsigned int *stage)
{
    
        *stage = 0;
        
        stop_mcp3553(address);

        delay_ms(100);
}


//derived from working test in setup.h
//2 part call; part 1 triggers start of reading on mcp3553
//part 2 reads the val out after conversion.
//as mcp3553 samples at below 60Hz and mcp3551 samples at below 16Hz
//you must let enough time pass between calls for this to work
//assumes good 50-100ms delay between stage0 & stage1 calls
//valid addresses {1,2,6,7}
BOOL run_mcp3553(unsigned int spiPort, unsigned int address, unsigned int *stage, long *adcVal)
{
    BOOL ok = FALSE;
    unsigned int notDataReady = 0;
    long val = -99999;

    if(*stage==0)
    {
        start_mcp3553(address);
        *stage = *stage + 1;

        return FALSE;
    }
    else if(*stage == 1)
    {
        if(spiPort == 1)
        {
            notDataReady = spi1_check_mcp3553();
        }
        else
        {
            notDataReady = spi2_check_mcp3553();
        }
        if(notDataReady>0)
        {
            return FALSE;
        }
        else
        {

            //take SPI readings and process data
            if(spiPort == 1)
            {
                val = spi1_process_mcp3553();
            }
            else
            {
                val = spi2_process_mcp3553();
            }
            
            stop_mcp3553(address);
            *stage = 0;

            if(val > 0)
            {
                *adcVal = val;
                return TRUE;
            }
            else
            {
                return FALSE;
            }
        }
    }
    else if(*stage > 1)
    {
        *stage = 0;
        stop_mcp3553(address);
    }

}





/*  old may not work
long readChannelData_mcp3553(unsigned int address)
{

//BRING !CS low
//	mPORTBClearBits(BIT_12);		//!CS  mcp3901
	mPORTBClearBits(BIT_7);		//!CS mcp3551
	delay_us(10);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope

	unsigned int not_dataReady;
do
{

	//read portB bit_11, sdi input (wait for mcp3553's !DR after !CS goes low)

	not_dataReady =  PORTReadBits(IOPORT_B, BIT_11);




}while(not_dataReady > 0);


	SPI1BUF = 0x03;
	while(!SPI1STATbits.SPIRBF);

	unsigned int mask = (~BIT_7 & 0xFF) & (~BIT_6 & 0xFF) & (~BIT_5 & 0xFF);
	unsigned int highByteCh0 = (SPI1BUF)&mask;
	delay_us(3);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


	SPI1BUF = 0x02;
	while(!SPI1STATbits.SPIRBF);

	unsigned int midByteCh0 = (SPI1BUF );
	delay_us(10);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


	SPI1BUF = (0x01);
	while(!SPI1STATbits.SPIRBF);

	unsigned int lowByteCh0 = SPI1BUF ;
	delay_us(3);	//delay is req'd when going slow, may not need it at full speed; will need to check with scope


//bring !CS High (idle state)
	//mPORTBSetBits(BIT_12);		//!CS  mcp3901
	mPORTBSetBits(BIT_7);		//!CS mcp3551


	unsigned int val = (highByteCh0<<8) + midByteCh0;
	unsigned int val2 = (midByteCh0<<8) + lowByteCh0;

//midByteCh0 = 0xFFFF;
//lowByteCh0 = 0xFFFF;

	unsigned int negative = BIT_4 & highByteCh0;

	long lval;

	if(negative > 0)
	{


		lval = ((highByteCh0&0xFFFFFFFF)<<16) + ((midByteCh0&0x0000FFFF)<<8) + lowByteCh0;

		lval = -(2097153 - lval);

	}
	else
	{
		lval = (highByteCh0<<16) + (midByteCh0<<8) + lowByteCh0;
	}



	return(lval);

}
*/
