//PID Controller.c
// Documentation: http://en.wikipedia.org/wiki/PID_controller/ and http://brettbeauregard.com/blog/2011/04/improving-the-beginners-pid-introduction/
#include "delay_time.h"  //has core timer features used herein to time loop

/* NOTES:  
 * needs to be called at regular intervals inside main fastest inner_loop
 *
 * To use:
 * 1) adjust PID parms by calling PID_SetTunings(..)
 * 2) init PID by calling PID_Initialize(..)
 *
 * 3 Loop { PID_Compute(..)  }
 *
 * if final destination req'd then loop till (Output - Setpoint < ep)
 */

#define bool unsigned short int
#define false 0
#define true 1

/*working variables*/
//unsigned long lastTime;
//double Input, Output, Setpoint;
//double ITerm, lastInput;
//double kp, ki, kd;
//int SampleTime = 10; //mSec per loop rate
//double outMin, outMax;
//bool inAuto = false;

#define MANUAL 0
#define AUTOMATIC 1

#define DIRECT 0
#define REVERSE 1

//int controllerDirection = DIRECT;


// ASSUMES called at regular intervals
//note: need to call setTunings to adjust ki, kp, kd to required sampleTime BEFORE using
//and need to call PID_Init(..) BEFORE using

//float PID_Compute(float Input, float Setpoint, float outMin, float outMax, float kp, float ki, float kd, float *ITerm, float *lastInput)
void PID_Compute(float Input, float Setpoint, float outMin, float outMax, float kp, float ki, float kd, float *ITerm, float *lastInput, float *Output)
{
    float dInput;


      /*Compute all the working error variables*/
      float error = Setpoint - Input;
      
      *ITerm += (ki * error);
      
      if(*ITerm > outMax)
          *ITerm= outMax;
      else if(*ITerm < outMin)
          *ITerm= outMin;

      dInput = (Input - *lastInput);

      /*Compute PID Output*/
      *Output = kp * error + *ITerm- kd * dInput;

      if(*Output > outMax)
          *Output = outMax;
      else if(*Output < outMin)
          *Output = outMin;


      *lastInput = Input;
      
      //return Output;
   
}

void PID_SetTunings(int SampleTime_ms, float *Kp, float *Ki, float *Kd, int controllerDirection)
{

   if (*Kp<0 || *Ki<0|| *Kd<0) return;

  float SampleTimeInSec = ((float)SampleTime_ms)/1000.0;
  
   *Kp = *Kp;
   *Ki = *Ki * SampleTimeInSec;
   *Kd = *Kd / SampleTimeInSec;

  if(controllerDirection ==REVERSE)
   {
      *Kp = (0 - *Kp);
      *Ki = (0 - *Ki);
      *Kd = (0 - *Kd);
   }
}


void PID_Initialize(float Input, float *lastInput, float Output, float *ITerm, float outMax, float outMin)
{
   *lastInput = Input;
   *ITerm = Output;

   if(*ITerm > outMax)
       *ITerm= outMax;
   else if(*ITerm < outMin)
       *ITerm= outMin;

   //lastTime = get_Time_ms();
}


/*
void PID_SetSampleTime(int NewSampleTime)
{
   if (NewSampleTime > 0)
   {
      double ratio  = (double)NewSampleTime
                      / (double)SampleTime;
      ki *= ratio;
      kd /= ratio;
      SampleTime = (unsigned long)NewSampleTime;
   }
}

void PID_SetOutputLimits(double Output, double Min, double Max, double *outMin, double *outMax)
{
   if(Min > Max) return;
   *outMin = Min;
   *outMax = Max;


   if(Output > outMax) 
       Output = *outMax;
   else if(Output < outMin) 
       Output = *outMin;

   if(ITerm > *outMax)
       ITerm = *outMax;
   else if(ITerm < outMin) 
       ITerm = *outMin;
}

void PID_SetMode(int Mode)
{
    bool newAuto = (Mode == AUTOMATIC);
    if(newAuto == !inAuto)
    {  //we just went from manual to auto
        PID_Initialize();
    }
    inAuto = newAuto;
}

void PID_SetControllerDirection(int Direction)
{
   controllerDirection = Direction;
}

*/

