//File:   SPI.H
#include <plib.h>

extern volatile unsigned int an_selA_val;   //analog select register A
extern volatile unsigned int an_selB_val;   //analog select register B

//cs3 (RB6, no analog)

//note: crktBd says spi2 but we're really using it as spi1 (pcduino bd)
void spi1_init(int use_cs0_rb15, int use_cs2_rb7, int use_cs3_rb6, int use_cs1_rb12)
{
    unsigned int configBits;
    //note: can't do this here or you have to read ANSELB and only change bits you need to while preserving rest!!!
//    ANSELB = (~BIT_11)|(~BIT_10)  ;	//must de-select all analog select bits for any digital I/O or digital function

ANSELB = an_selB_val & (~BIT_13) & (~BIT_14)& (~BIT_15);	//must de-select all analog select bits for any digital I/O or digital function
an_selB_val = ANSELB;

    configBits = ANSELB;
    
//    ANSELB = ANSELB & ~BIT_13 & ~BIT_14;


    if(use_cs0_rb15)
    {
        ANSELB = ANSELB & ~BIT_15;  //only 1 of 3 cs lines that is also analog
        configBits = ANSELB;

        PORTSetPinsDigitalOut(IOPORT_B, BIT_15);	//CS0; RB15, An9
        mPORTBSetBits(BIT_15);		//!CS0 	//for mcp3553
    }

    if(use_cs1_rb12)
    {
        ANSELB = ANSELB & ~BIT_12;  //only 1 of 3 cs lines that is also analog
        configBits = ANSELB;

        PORTSetPinsDigitalOut(IOPORT_B, BIT_12);	//CS0; RB15, An9
        mPORTBSetBits(BIT_12);		//!CS0 	//for mcp3553
    }

    PORTSetPinsDigitalOut(IOPORT_B, BIT_10);	//!CS1, !CS2  no analog
    PORTSetPinsDigitalOut(IOPORT_B, BIT_15);	//!CS1, !CS2  has analog
    mPORTBSetBits(BIT_10);  //idle high
    mPORTBSetBits(BIT_15);  //idle high
    
    PORTSetPinsDigitalOut(IOPORT_B, BIT_13);	//SDO1; RB13/AN11  analog
    PORTSetPinsDigitalOut(IOPORT_B, BIT_14);	//sck1; RB14/AN10  analog
    PORTSetPinsDigitalIn(IOPORT_B, BIT_11);	//SDI1; RB11, No analog

    mPORTBClearBits(BIT_13);	//SDO-1 (pin 24)


    //test only
    mPORTBClearBits(BIT_10);  //idle high  !CS1
    mPORTBSetBits(BIT_10);  //idle high
    mPORTBClearBits(BIT_10);  //idle high  !CS1
    mPORTBSetBits(BIT_10);  //idle high

    mPORTBClearBits(BIT_15);  //idle high  !CS2
    mPORTBSetBits(BIT_15);  //idle high
    mPORTBClearBits(BIT_15);  //idle high  !CS2
    mPORTBSetBits(BIT_15);  //idle high

    mPORTBSetBits(BIT_13);   //idle low SDO
    mPORTBClearBits(BIT_13);
    mPORTBSetBits(BIT_13);
    mPORTBClearBits(BIT_13);  //idle low SDO

    mPORTBSetBits(BIT_14);   //SCK
    mPORTBClearBits(BIT_14);
    mPORTBSetBits(BIT_14);
    mPORTBClearBits(BIT_14);  //SCK


    if(use_cs2_rb7)
    {
        PORTSetPinsDigitalOut(IOPORT_B, BIT_7);	//CS0; RB15, An9
        mPORTBSetBits(BIT_7);		//!CS0 	//for mcp3553
    }

    if(use_cs3_rb6)
    {
        PORTSetPinsDigitalOut(IOPORT_B, BIT_6);	//CS0; RB15, An9
        mPORTBSetBits(BIT_6);		//!CS0 	//for mcp3553
    }


//SETUP SPI-1	//NOTE: SCK-1 is RB15
	SDI1Rbits.SDI1R = 3;	//SET RPB11 to SDI1
	RPB13Rbits.RPB13R = 3;	//SET RPB13 to SDO1




        
//	SPI1CON = 0x8130 + 64;	//config SPI-2; 8 bit master, CKE=1, CKP=1;	mode 1,1

        //added this; hopefully it'll help
        SPI1CON2 = 0x0000;  //make sure auden == 0 (to ensure 8bit mode)

        //this seemes to have worked for mcp3553; also seems correct by data-sheet; do need to check SPxCON2
	SPI1CON = 0x8120 ;	// config SPI-2; 8 bit master, CKE=1, CKP=0; mode 0,0


//	SPI2BRG = 0x0006;	//9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1)); sets SPI clock = 533kHz/byte
//	SPI2BRG = 12;	//9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1)); sets SPI clock = 266kHz/byte

	//for testing with LogicStudio16; (like 10 which gives a 2MHz clock, but too fast for the MCP chip)
	//SPI1BRG = 10;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

	//for testing with LogicStudio16; (like 10 which gives a 2MHz clock, but too fast for the MCP chip)
	//SPI1BRG = 35;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

	SPI1BRG = 528;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

	//for testing with Logic16; use very slow speed to avoid crashing Logic16 (like 700 which gives a 25kHz clock)
//	SPI1BRG = 700;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

}

//for Deutsch HV precAn bd
void spi2_init(int use_cs0_rb15, int use_cs2_rb7, int use_cs3_rb6, int use_cs1_rb12)
{
    unsigned int configBits;
    //note: can't do this here or you have to read ANSELB and only change bits you need to while preserving rest!!!
//    ANSELB = (~BIT_11)|(~BIT_10)  ;	//must de-select all analog select bits for any digital I/O or digital function

    configBits = ANSELB;

    ANSELB = ANSELB & ~BIT_13 & ~BIT_14;

    configBits = ANSELB;

    if(use_cs0_rb15)
    {
        ANSELB = ANSELB & ~BIT_15;  //only 1 of 3 cs lines that is also analog
        configBits = ANSELB;

        PORTSetPinsDigitalOut(IOPORT_B, BIT_15);	//CS0; RB15, An9
        mPORTBSetBits(BIT_15);		//!CS0 	//for mcp3553
    }

    if(use_cs1_rb12)
    {
        ANSELB = ANSELB & ~BIT_12;  //only 1 of 3 cs lines that is also analog
        configBits = ANSELB;

        PORTSetPinsDigitalOut(IOPORT_B, BIT_12);	//CS0; RB15, An9
        mPORTBSetBits(BIT_12);		//!CS0 	//for mcp3553
    }



    PORTSetPinsDigitalOut(IOPORT_B, BIT_11);	//SDO1; RB11/ no An
    PORTSetPinsDigitalOut(IOPORT_B, BIT_14);	//sck1; RB14/AN10
    PORTSetPinsDigitalIn(IOPORT_B, BIT_13);	//SDI1; RB13, An11

    //mPORTBClearBits(BIT_11);	//SDO-1 (pin 24)


    if(use_cs2_rb7)
    {
        PORTSetPinsDigitalOut(IOPORT_B, BIT_7);	//CS0; RB15, An9
        mPORTBSetBits(BIT_7);		//!CS0 	//for mcp3553
    }

    if(use_cs3_rb6)
    {
        PORTSetPinsDigitalOut(IOPORT_B, BIT_6);	//CS0; RB15, An9
        mPORTBSetBits(BIT_6);		//!CS0 	//for mcp3553
    }


    //test RB11, RB14, RB12, RB6
    mPORTBClearBits(BIT_14);
    mPORTBClearBits(BIT_11);
    mPORTBClearBits(BIT_12);
    mPORTBClearBits(BIT_15);

    mPORTBSetBits(BIT_14);
    mPORTBSetBits(BIT_11);
    mPORTBSetBits(BIT_12);
    mPORTBSetBits(BIT_15);

    mPORTBClearBits(BIT_14);
    mPORTBClearBits(BIT_11);
    mPORTBClearBits(BIT_12);
    mPORTBClearBits(BIT_15);

    mPORTBSetBits(BIT_14);
    mPORTBSetBits(BIT_11);
    mPORTBSetBits(BIT_12);
    mPORTBSetBits(BIT_15);

//SETUP SPI-1	//NOTE: SCK-1 is RB15
	//SDI1Rbits.SDI1R = 3;	//SET RPB11 to SDI1
	//RPB13Rbits.RPB13R = 3;	//SET RPB13 to SDO1


        SDI2Rbits.SDI2R = 3;	//SET RPB13 to SDI2
        RPB11Rbits.RPB11R = 4;	//SET RPB11 to SDO2


//	SPI1CON = 0x8130 + 64;	//config SPI-2; 8 bit master, CKE=1, CKP=1;	mode 1,1

        //added this; hopefully it'll help
        SPI2CON2 = 0x0000;  //make sure auden == 0 (to ensure 8bit mode)

        //this seemes to have worked for mcp3553; also seems correct by data-sheet; do need to check SPxCON2
	SPI2CON = 0x8120 ;	// config SPI-2; 8 bit master, CKE=1, CKP=0; mode 0,0


//	SPI2BRG = 0x0006;	//9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1)); sets SPI clock = 533kHz/byte
//	SPI2BRG = 12;	//9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1)); sets SPI clock = 266kHz/byte

	//for testing with LogicStudio16; (like 10 which gives a 2MHz clock, but too fast for the MCP chip)
	//SPI1BRG = 10;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

	//for testing with LogicStudio16; (like 10 which gives a 2MHz clock, but too fast for the MCP chip)
	//SPI1BRG = 35;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

	SPI2BRG = 528;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

	//for testing with Logic16; use very slow speed to avoid crashing Logic16 (like 700 which gives a 25kHz clock)
//	SPI1BRG = 700;	//(good for fast spi)  9 bit divider; Fsck = Fpb/(2*SPI2BRG + 1));

}