//adc.h
#include <p32xxxx.h>
#include <plib.h>


void initADC_autoSample()
{

	AD1CON1bits.ADON = 0;	//turn off ADC; first thing you do

	AD1CON1bits.SIDL = 1;	//if SIDL==1 then discontinue module operation when running in idle mode
	AD1CON1bits.FORM = 0;	// if FORM==0, use integer 16 bit output (10 bit unsigned data)
	AD1CON1bits.CLRASAM = 0;	//if CLRASM == 0; normal op. buffer contents will be over-written on next conversion
	AD1CON1bits.SSRC = 0x07;	//if SSRC = 111; internal counter ends sampling and starts conversion (auto convert mode)	
	//AD1CON1bits.ASAM = 1;		//AUTO start sampling mode by setting AD1CON1bits.SAMP bit; check for done with (!IFS1 & 0x0002) 
	AD1CON1bits.ASAM = 0;		//manual start sampling mode by setting AD1CON1bits.SAMP bit; check for done with AD1CON1bits.DONE 

	AD1CON2bits.VCFG = 0;		//if VCFG==0; use AVDD & AVSS for reference voltages
	AD1CON2bits.OFFCAL = 0; 	//if OFFCAL==0; disable offset calibration mode
	
	//AD1CON2bits.CSCNA = 1; 	//if CSCNA==0; scan inputs
	AD1CON2bits.CSCNA = 0; 	//if CSCNA==0; do not scan inputs

	AD1CSSL = 0x0000;	//scan channel select mask; if set == 0, then no scan


	AD1CON2bits.BUFM = 0; 	//if BUFM==0; config as one 16-word buffer 
	AD1CON2bits.BUFS = 0; 	//if BUFS==0; only valid when BUFM=1 (ADRES SPLIT INTO 2x8 word buffers); tells which buffer is being filled
	
	//AD1CON2bits.SMPI = numChannels; 	//if SMPI=#; writes each conversion to ADC1BUF0-N starting with BUF0 (FOR SCAN MODE)
	AD1CON2bits.SMPI = 0; 	//if SMPI=0; only writes to the 1st buffer location; ADC1BUF0
							//SMPI sets how many buffer locations are written to; so if scanning it should match the # of samples scanned;
							//also sets how many samples are taken before an interrupt is fired; if =0 then fires after every conversion 

	AD1CON2bits.ALTS = 0; 	//if ALTS==0; do not alternate between multiplexors on each subsequent sample 

	AD1CON3bits.ADRC = 0;	//if ADRC=0; derive clk from PBCLK
//set to 200kHz sampling rate
	AD1CON3bits.SAMC = 4;	//if SAMC=7; use 7 TAD sample clock tics to sample signal; (so sample-time = 1400ns)

	//AD1CON3bits.ADCS = 2;	//if ADCS=2; ADC_CLKDIV = 8;  so TAD = (ADCS+1) 2 * Tpb = 200ns (so good to 5kOhm input impedance)	
	AD1CON3bits.ADCS = 1;	//if ADCS=2; ADC_CLKDIV = 2;  so TAD = (ADCS+1) 2 * Tpb = 50ns (so good to 5kOhm input impedance)	

	AD1CHSbits.CH0NB = 0;	//if CH0NB=0; use Vr- as negative input for MUX-B (if 1 use AN1)
	AD1CHSbits.CH0NA = 0;	//if CH0NA=0; use Vr- as negative input for MUX-A (if 1 use AN1)

	AD1CHSbits.CH0SB = 0;	//if CH0SB=#; # is which AN# channel # MUX-B is selected for
	AD1CHSbits.CH0SA = 0;	//if CH0SA=#; # is which AN# channel # MUX-A is selected for

	
	//NOTE: AD1PCFG is not avail. in all pic32 chips; check data-sheet to see if it exists (does exist in Olimex; not in pic32mx120
	//in pic32mx120 AD1PCFG has been replaced by ANSELx which has already been setup via 	initPorts(); as it has to be cleared to enable DIO
		
	//AD1PCFG = ~mask;		//Olimex bd only: bit position corresponds to analog channel & a "0" cleared bit sets it to an analog input; otherwise digital 


	//AD1CSSL = ~mask;		//ADC Input Scan Select Register; bit position corresponds to analog channel & a "1" set bit includes corresponding ANx channel in input scan
							//must have setup AD1CON2bits.CSCNA=1 to scan inputs for this to work

//note: may not have to do this as we're not scanning channels "CSCNA=0"; need to check if necessary
/*
	AD1CSSL = 0x0000;		//start out with no channels selected

	if(analog_ch_mask[0] == 1)	//check if set as active Analog channel
	{
		AD1CSSL = AD1CSSL & analogCh1;	//set high to select analog channel
	}

	if(analog_ch_mask[1] == 1)	//check if set as active Analog channel
	{
		AD1CSSL = AD1CSSL & analogCh2;
	}

	if(analog_ch_mask[2] == 1)	//check if set as active Analog channel
	{
		AD1CSSL = AD1CSSL & analogCh3;
	}

	if(analog_ch_mask[3] == 1)	//check if set as active Analog channel
	{
		AD1CSSL = AD1CSSL & analogCh4;
	}
	

PORTSetPinsDigitalIn(IOPORT_A, BIT_0);
PORTSetPinsDigitalIn(IOPORT_A, BIT_1);
//PORTSetPinsDigitalIn(IOPORT_B, BIT_0);
//PORTSetPinsDigitalIn(IOPORT_B, BIT_1);

AD1CSSL = BIT_0 | BIT_1  ;
*/


	AD1CON1bits.ADON = 1;	//turn on ADC; last thing you do

}

void set_ADC_channel(int channelNum)
{
	AD1CHSbits.CH0SA = channelNum;
}

void trigger_ADC()
{
	AD1CON1bits.SAMP = 1;	//start sampling
}

//assumes you've already called both set_ADC_channel() & trigger_ADC()
//non blocking; returns 1 if adc can be read, returns 0 if it's not ready i.e. it can't 
unsigned int read_preTriggered_ADC(unsigned int* data)
{
	if(AD1CON1bits.DONE)
	{
		*data = ADC1BUF0;
		return(1);
	}
	else
	{
		*data = 0;
		return(0);
	}

}


/*
unsigned int readADC_ManualStart_AutoConvert(int ch)
{
   int i, j, k;
   unsigned int samp[16];
   unsigned int total = 0;
   unsigned int channelNum = 0;
//  MANUAL START; AUTO CONVERSION
	//AD1CHS = 0x000C0000; // Connect RB12/AN12 as CH0 input
	// in this example RB12/AN12 is the input

	//this works; but you need correct channel which we've modified per #defines to fit our output board map
//	AD1CHSbits.CH0SA = ch;

//so remap here
	if(ch == 1)
		channelNum = analogCh1;
	else if(ch == 2)
		channelNum = analogCh2;
	else if(ch == 3)
		channelNum = analogCh3;
	else if(ch == 4)
		channelNum = analogCh4;
	else
		return(999999);


	AD1CHSbits.CH0SA = channelNum;
	delay_usecs(20);
	//delay_usecs(100);

	//AD1CON1CLR = 0x0002; // start sampling then...


	for(j=0;j<16;j++)
	{
		AD1CON1bits.SAMP = 1;	//start sampling
		for(i=0;i<1;i++)
			asm("nop");
		// after 31Tad go to conversion
		while (!AD1CON1bits.DONE); // conversion done?
		//for(i=0;i<6;i++)
		//	asm("nop");

		samp[j] = ADC1BUF0;

		//if(samp < 400)
		//{
		//	k = j;
		//}

		total += samp[j];
		
		//for(i=0;i<10;i++)
		//	asm("nop");
	}



	total = total >> 2;

	return((unsigned int)(total));

}
*/