#include <plib.h>
#include <string.h>

//#include "dataLog.h"
#include "mpu9150_driver.h";
#include "mag3110_compass.h"
#include "mcp3553_driver.h"


//#include "uart.h"
extern char* pTxBuf;

float run_ADC(int adcNum, char* sData, int id);

const int VERBOSE = 1;

enum OP_TYPES {SKIP, SW1, SW2, LV_OUT1, PWM1, PWM2,
                US1, US2, PADC1, PADC2, PADC3,
                PADC4, ADC1, ADC2, TC1, TC2,
                BARO, HEADING, TILT_XY, TILT_XZ, TILT_YZ,
                ANGLE_X, ANGLE_Y, ANGLE_Z, GYRO_X, GYRO_Y,
                GYRO_Z, LOOP_COUNT, PROGRAM1, PROGRAM2, DUTY_CYCLE1,
                DUTY_CYCLE2, DURATION_1_ms, DURATION_2_ms, Period_SW1, Period_SW2 };
//NOTE: PROGRAM1 AND PROGRAM2 RUN BEFORE TASK LOOP STARTS

//Precision ADC correction coef's
//Harvester: board #1 (chassis)
#define board_3

//#ifdef board_1
//float PADC1_SLOPE = 0.015508;
//float PADC2_SLOPE = 0.0155;
//float PADC1_INTERCEPT = -0.000270;
//float PADC2_INTERCEPT = 0.0004;
//#elif board_2
//#ifdef board_2
//float PADC1_SLOPE = 0.015537;
//float PADC2_SLOPE = 0.0155;
//float PADC1_INTERCEPT = -0.000363;
//float PADC2_INTERCEPT = 0.0004;
#ifdef board_3
float PADC1_SLOPE = 0.015545;
float PADC2_SLOPE = 0.015508;
float PADC1_INTERCEPT = -0.002;
float PADC2_INTERCEPT = 0.00;
#else
//Harvester: board #2 (basket)
float PADC1_SLOPE = 0.0155;
float PADC2_SLOPE = 0.0155;
float PADC1_INTERCEPT = 0.0018;
float PADC2_INTERCEPT = 0.0004;
#endif


//loop_cnt is # times to run TASK_LOOP sampling routine; if == -1, then continuous loop
//if loop_cnt == 0 stop we're done
int loop_cnt = 0;
int duty_cycle1 = 80;  //used by Program1 & Program2; set IN configDataLogger(..) via DUTY_CYCLEx OP_TYPES
int duty_cycle2 = 55;
int duration1 = 0;  //# ms program1 should run for on first timing loop
int duration2 =0;   //# ms program1 should run for on second timing loop
BOOL PROGRAM1_Enabled = FALSE;
BOOL PROGRAM2_Enabled = FALSE;

unsigned int timeOutDuration_ms = 30000;

unsigned int ADC1_Chan = 9;  //RB15
//unsigned int ADC2_Chan = 12; //RB12
unsigned int ADC2_Chan = 10; //RB14 (sck2 on schematic pic32_click_duino_expansion_lowNoise__4_16_14)



//list of installed functions 1:1 correspondance with OP_TYPES above
BOOL Installed_Ops[35] = {FALSE, FALSE, FALSE, FALSE, FALSE,
                          FALSE, FALSE, FALSE, FALSE, FALSE,
                          FALSE, FALSE, FALSE, FALSE, FALSE,
                          FALSE, FALSE, FALSE, FALSE, FALSE,
                          FALSE, FALSE, FALSE, FALSE, FALSE,
                          FALSE, FALSE, FALSE, FALSE, FALSE,
                          FALSE, FALSE, FALSE, FALSE, FALSE};


BOOL SamplingOps[15];  //what we're actually reading




struct OP
{
    int ParmID;
    int ParmTypeNum;  //same thing as SAMPLING_OP, id of what we're using; # version of OpType below (use for comm back to PC
    enum OP_TYPES ParmType;
    int ParmFreq;
    int RValType;  // what type of return value this device provides: {0: none, 1:int, 2:float}
    int cur_sw_state;  //if switch, this will hold the status of the last reading of that switch
    float cur_fVal;     //if sensor, holds last reading of sensor (used by pid control loop so it doesn't have to re-read it)
} innerLoop_ops[15], middleLoop_ops[15], outerLoop_ops[15];
//record of config types; holds {id, parmType, parmFreq}


//{TC1, padc1, padc2, tiltXY, angleX, angleY, angleZ, magX, magY, magZ, sw1, sw2, LED1, relay2, relay3, pwm1, pwm2}

struct PID
{
    struct OP *inputOp_ptr;
    struct OP *outputOp_ptr;
    int inputOp_id;  //ptr to an existing OP (input variable (sensor)
    enum OP_TYPES input_opType;
    int outputOp_id;  //ptr to an existing OP (output variable (what we're actuating)
    enum OP_TYPES output_opType;
    float kp;
    float kd;
    float ki;
    int control_direction;
    int control_state; //on_off {1,0}
    float ITerm;    //integral sum
    float lastInput;  //last input for derivative calc
    float outMin;
    float outMax;
    float setPoint;
    int reset_tmr_tic_cnt;
}pid1, pid2;


BOOL dataLogSetup = TRUE;   //Start with setup = TRUE and use default

BOOL GetHeading = FALSE;

BOOL GetBaroPressure = FALSE;

BOOL GetTilt_XY = FALSE;
BOOL GetTilt_XZ = FALSE;
BOOL GetTilt_YZ = FALSE;

BOOL GetAccel_XYZ = FALSE;
BOOL GetGyro_XYZ = FALSE;

BOOL GetPrecAdc_1 = FALSE;
BOOL GetPrecAdc_2 = FALSE;
BOOL GetPrecAdc_3 = FALSE;
BOOL GetPrecAdc_4 = FALSE;

BOOL GetSwitch_1 = FALSE;
BOOL GetSwitch_2 = FALSE;


const int maxNumParms = 40;  //was 15; but needed to add a few so we could pass in extra vars for Loop_Count, DutyCycle ect
int configParms[80];  //should be 2x maxNumParms

float configPID_Parms[80];  //should be 2x maxNumParms


//hard coded loop freqs  (must be multiple of each other so loops are nested
const int outer_loopFreq = 1;
const int middle_loopFreq = 10;
const int inner_loopFreq = 100;



int innerLoop_setupCntr = 0;  //used to count total # in this group
int middleLoop_setupCntr = 0;
int outerLoop_setupCntr = 0;


int loopCount = -1;  //used to keep track of where we're at;  main loop uses this # to see if we should be running middle or outer loop funs (loopcount % 10 or %100)

//how often we update data readings
int loopDelay_ms = 1000;


int pwm_reset_timer_val = 0x3E8;


/* In Oder to make this work generically; datalogger has to be pre-configured with devices tied to specific ports;

 * Given this, need somewhere where we pre-define said ports...


 * portA: RA0(pullUp), RA1(pullup)
 * portB: RB3, RB6, RB7, RB12, RB15
 *
 *      PWM OC1: {RA0, RB3, RB7, RB12, RB15}
 *      PWM OC2: {RA1}
 *      PWM OC3: {}
 *      PWM OC4: {RB2, RB6}
 *      PWM OC5: {RB2, RB6}
 *
 *      An:  {RA0(An0), RA1(An1), RB2(An4), RB3(An5), RB12(An12), RB15(An9}
 *
 *      !Cs: {RB3(cs6), RB6(cs3), RB7(cs2), RB15(cs0)}
 * 
 *      not avail this bd. use pwm2    LED1 is on PORTB.RB12
 *          SW1  is on PORTA.RA0,
 *          SW2  is on PORTA.RA1,
 *          PWM1 is on PORTB.RB3(OC1),  [An5]
 *          PWM2 is on PORTB.RB6(OC4),
 *          UltraSonic1: pic_rx1  (Alt Comm: On different pin than normal rx1)
 *          UltraSonic2: pic_rx2
 *          Precision Adc1: spi RB15(cs0)
 *          Precision Adc1: spi RB7(cs2)
 *          ADC1: RB15(an9)
 *          ADC2:
 *
 *          alternative; use on-board adc for US1
 *          alt US1 (on alt comm1 connector): !CS0 RB15 (an9)
 *
 *
 *          config from PC should allow for multiple use on pins...
 *          one way to accomplish this is to not init pins to digital
 *          until they're config'd by pc, thus they default to analog inputs
 *          example: pwm1 (RB3[An5]), let the pwm config convert from analog to
 *          digital

 *          similarly, could do this with spi port (if not using Prec ADC, no need
 *          for the spi port and it frees up:
 *
 *          //note: spi pins avail on JP2 header
 *          //sck2 RB14 (an10)
 *          //sdo2 RB13 (an11; could be alt Comm1)
 *          //sdi2 RB11 (no analog)
 *          //!CS0 RB15 (an9)
 *          //!CS5 RB12 (an12)
 *          //!cs1 RB10 (no analog)
 *          //!cs2 RB7  (no analog)
 *          //!cs3 RB6  (no analog; could be alt Comm1)
 *          //!cs6 RB3  (no analog)
 *          //!cs4 RA2  (no analog)
 *
 */


//hard-code hardware directly into funcs?  not great...
unsigned int last_sw1 = 0;
unsigned int last_sw1_state = 0;
unsigned int last_sw2 = 0;
unsigned int last_sw2_state = 0;

unsigned int last_adc_chan = 0;
//delay_ms(100);
unsigned int sw1_configured = 0;
unsigned int sw2_configured = 0;


//use for Period measurement
unsigned int toggle1_cnt = 0;  //keeps track of # of toggles, sw1, since last re-start of timer45
unsigned int toggle2_cnt = 0;  //keeps track of # of toggles, sw2, since last re-start of timer45

unsigned long total_tics1 = 0;
unsigned long total_tics2 = 0;

unsigned int timer_sw1_tic_cnt0 = 0;  //how many tics between toggles (latest)
unsigned int timer_sw1_tic_cnt1 = 0;  //how many tics between toggles (2nd latest)
unsigned int timer_sw1_tic_cnt2 = 0;  //how many tics between toggles (3rd latest)
unsigned int timer_sw1_tic_cnt3 = 0;  //how many tics between toggles

unsigned int timer_sw2_tic_cnt0 = 0;  //how many tics between toggles (latest)
unsigned int timer_sw2_tic_cnt1 = 0;  //how many tics between toggles (2nd latest)
unsigned int timer_sw2_tic_cnt2 = 0;  //how many tics between toggles (3rd latest)
unsigned int timer_sw2_tic_cnt3 = 0;  //how many tics between toggles


//ISR TMR4 setvars to keep track of sw1, sw2 freq
extern volatile unsigned int period1;  //not a true period, more of a count
extern volatile unsigned int period2;


//takes at least 2 high readings in a row to make a transition
unsigned char readSwitch(int switchNum)
{
    unsigned int val, data, configBits, tic_1, tic_2;

    if(switchNum == 1)  // RA0 (has pullup on it)
    {
        if(sw1_configured == 0)
        {
            configBits = ANSELB;
            ANSELB = ANSELB & ~BIT_0;
            configBits = ANSELB;

            PORTSetPinsDigitalIn(IOPORT_B, BIT_0);

            sw1_configured = 1;
        }
        
        //val = PORTReadBits(IOPORT_A, BIT_0 | BIT_4);
        val = PORTReadBits(IOPORT_B, BIT_0);

        if(val > 0)
            val = 1;
        else
            val = 0;


        //do debounce here; assume last call was long enough to go to ensure stable state
        if(val == last_sw1)
        {
            if(last_sw1_state != val)
            {
                if(val == 1)
                {

                    toggle1_cnt++;
                    
                    tic_1 = TMR4;
                    //delay_ms(1);
                    //tic_2 = TMR4;

                    //total_tics1 += tic_1;

                    //TMR4 = 0;

                    if((tic_1 - timer_sw1_tic_cnt0) > 0)
                    {
                        timer_sw1_tic_cnt3 = timer_sw1_tic_cnt2;
                        timer_sw1_tic_cnt2 = timer_sw1_tic_cnt1;
                        timer_sw1_tic_cnt1 = timer_sw1_tic_cnt0;
                        timer_sw1_tic_cnt0 = TMR4 ;

                        //TMR4 = 0;
                    }
                }
            }
            
            last_sw1_state = val;  //transition switch state, had 2 consequitive readings
            last_sw1 = val;         //update last sw reading

            
            data = val;

        }
        else
        {
            last_sw1 = val;         //update last sw reading
            data = last_sw1_state;  //send current switch state (don't transition yet)
        }

    }
    else if(switchNum == 2)  //RA1 (has pullup on it)
    {
        if(sw2_configured == 0)
        {
            configBits = ANSELB;
            ANSELB = ANSELB & ~BIT_3;
            configBits = ANSELA;

            PORTSetPinsDigitalIn(IOPORT_B, BIT_3);

            sw2_configured = 0;
        }

        val = PORTReadBits(IOPORT_B, BIT_3);

        if(val > 0)
            val = 1;
        else
            val = 0;


        //do debounce here; assume last call was long enough to go to ensure stable state
        if(val == last_sw2)
        {
            if(last_sw1_state != val)
            {
                if(val == 1)
                {
                    toggle2_cnt++;
                    timer_sw2_tic_cnt3 = timer_sw2_tic_cnt2;
                    timer_sw2_tic_cnt2 = timer_sw2_tic_cnt1;
                    timer_sw2_tic_cnt1 = timer_sw2_tic_cnt0;
                    timer_sw2_tic_cnt0 = TMR4;

                    TMR4 = 0;
                }
            }

            last_sw2_state = val;  //transition switch state, had 2 consequitive readings
            last_sw2 = val;         //update last sw reading

            data = val;

        }
        else
        {
            last_sw2 = val;         //update last sw reading
            data = last_sw2_state;  //send current switch state (don't transition yet)
        }
    }


    return data;

}

//set LED or relays (pwm's being used as simple outputs)
BOOL setRelay(int relayNum, int state)
{


    if(relayNum == 1)  //LED1: RB12
    {
        if(state==0)
        {
             mPORTBClearBits(BIT_12);

        }
        else
        {
             mPORTBSetBits(BIT_12);
        }
    }
    else if(relayNum == 2) //PWM1: RB3
    {
        if(state==0)
        {
             mPORTBClearBits(BIT_3);

        }
        else
        {
             mPORTBSetBits(BIT_3);
        }
    }
    else if(relayNum == 3) //PWM2: RB6
    {
        if(state==0)
        {
             mPORTBClearBits(BIT_6);

        }
        else
        {
             mPORTBSetBits(BIT_6);
        }
    }


}


//returns TMR_Tick (reset roll-over val
int calc_pwm_reset(int preScalar, int pwmFreq)
{
    float sysFreq = 40000000.0;
    float pbDiv = 1.0;
    float fResetVal = sysFreq/pbDiv / (float)preScalar / (float)pwmFreq;

    int reset = (int)fResetVal;

    return reset;
}



//need to share timer; assume 16bit timer is fine

int initPWM_Output(int pwmNum, int pwmFreq)
{

    int preScalar = 256; //used fixed preScalar

    int reset = calc_pwm_reset(preScalar, pwmFreq);

    //open TMR2 with prescalar and set roll-over val
    OpenTimer2(T2_ON | T2_PS_1_256, reset);   //open TMR2 ( with 1000 reset value
    
    //set PWM up with 0 duty-cycle
    if(pwmNum==1)
    {
        //MAP OC4 to RB6
        //RPB6Rbits.RPB6R = 5; //SET RPB6 to OC4
        RPB1Rbits.RPB1R = 5; //SET RPB1 to OC2

        /* Open Timer2 with Period register value */
    //OpenTimer2(T2_ON| T2_PS_1_256, 0x550);

    /* Enable OC | 32 bit Mode  | Timer2 is selected | Continuous O/P   | OC Pin High , S Compare value, Compare value*/
//    OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , 0x550, 0x500 );


        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
        OpenOC2( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
    }
    else if(pwmNum==2)
    {
        //MAP OC4 to RB6
        //RPB6Rbits.RPB6R = 5; //SET RPB6 to OC4
        RPB2Rbits.RPB2R = 5; //SET RPB2 to OC4

        /* Open Timer2 with Period register value */
    //OpenTimer2(T2_ON| T2_PS_1_256, 0x550);

    /* Enable OC | 32 bit Mode  | Timer2 is selected | Continuous O/P   | OC Pin High , S Compare value, Compare value*/
//    OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , 0x550, 0x500 );


        OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
        //OpenOC2( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
    }
    else if(pwmNum==3)
    {
        //MAP OC4 to RB6
        //RPB6Rbits.RPB6R = 5; //SET RPB6 to OC4
        RPB4Rbits.RPB4R = 5; //SET RPB4 to OC1

        /* Open Timer2 with Period register value */
    //OpenTimer2(T2_ON| T2_PS_1_256, 0x550);

    /* Enable OC | 32 bit Mode  | Timer2 is selected | Continuous O/P   | OC Pin High , S Compare value, Compare value*/
//    OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , 0x550, 0x500 );


        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
        OpenOC1( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
    }
    else if(pwmNum==4)
    {
        //MAP OC4 to RB6
        //RPB6Rbits.RPB6R = 5; //SET RPB6 to OC4
        RPA4Rbits.RPA4R = 6; //SET RPB4 to OC5

        /* Open Timer2 with Period register value */
    //OpenTimer2(T2_ON| T2_PS_1_256, 0x550);

    /* Enable OC | 32 bit Mode  | Timer2 is selected | Continuous O/P   | OC Pin High , S Compare value, Compare value*/
//    OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , 0x550, 0x500 );


        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
        OpenOC5( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-1 );
    }
    else
    {
        //MAP OC1 to RB7
        RPB7Rbits.RPB7R = 5; //SET RPB7 to OC1
        OpenOC1( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-10 );
    }

    return reset; //for use by pwmSet routine

    
    // Open Timer2 with Period register value
    //OpenTimer2(T2_ON, 0x550);

    //Output Compare is mappable, so also need to set that


    // Enable OC | 32 bit Mode  | Timer2 is selected | Continuous O/P   | OC Pin High , S Compare value, Compare value
    //OpenOC1( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , 0x550, 0x500 );
    /*
      The expected output looks like the diagram below with approximately 6% duty-cycle

           ||      ||      ||
     ______||______||______||______

    */


}

//NOT USED!!!!
//duty is from 0-100
//solid state relay inverts pwm signal; so calc flips from 100-duty to duty
BOOL setPWM_SolidStateRelay_Output(int pwmNum, int reset, int duty)
{
    int iDuty;
    //float fDuty = (float)(100-duty)/100.0;
    float fDuty = (float)(duty)/100.0;


    if(duty <= 0)
    {
        iDuty = 0;
    }
    else if(duty >= 100)
    {
        iDuty = reset - 1;
    }
    else
    {
        //fDuty = fDuty - 0.05;  //correction for solid-state relay
        fDuty = fDuty - 0.005;  //correction for solid-state relay
        fDuty = fDuty * reset;

        iDuty = (int)fDuty;
    }

    

    if(pwmNum == 1)
    {
        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-100 );
        OpenOC2( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }
    else if(pwmNum == 2)
    {
        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-100 );
        OpenOC4( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }
    else if(pwmNum == 3)
    {
        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-100 );
        OpenOC1( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }
    else if(pwmNum == 4)
    {
        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-100 );
        OpenOC5( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }
    else //pwm5
    {
        OpenOC3( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }

    return (TRUE);
}

//duty is from 0-100
BOOL setPWM_Output(int pwmNum, int reset, int duty)
{
    int iDuty;
    float fDuty = (0.9248*(float)duty+8.0);
    fDuty = (100.0-fDuty)/100.0;


    if(fDuty <= 0.01)
    {
        fDuty = 0.01;
        
        fDuty = fDuty * reset + 0.5;  //add 0.5 to force round instead of truncate

        iDuty = (int)fDuty;
    }
    else if(fDuty >= 0.5)  //turn off solenoid
    {
        iDuty = reset - 100;
    }
    else
    {
        fDuty = fDuty * reset + 0.5;  //add 0.5 to force round instead of truncate

        iDuty = (int)fDuty;
    }



    if(pwmNum == 1)
    {
        //OpenOC4( OC_ON | OC_TIMER_MODE32 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, reset-100 );
        OpenOC4( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }
    else //pwm2
    {
        OpenOC1( OC_ON | OC_TIMER_MODE16 | OC_TIMER2_SRC | OC_CONTINUE_PULSE | OC_LOW_HIGH , reset, iDuty );
    }

    return (TRUE);
}




/*
enum OP_TYPES {SW1, SW2, LV_OUT1, PWM1, PWM2,
                US1, US2, PADC1, PADC2, PADC3,
                PADC4, ADC1, ADC2, TC1, TC2,
                BARO, HEADING, TILT_XY, TILT_XZ, TILT_YZ,
                ANGLE_X, ANGLE_Y, ANGLE_Z, GYRO_X, GYRO_Y,
                GYRO_Z};
*/

// SHOULD WE HAVE EACH OF THESE RETURN A DOUBLE (ENGR UNITS) OF READ VALUE?
// BE NICE FOR PID CONTROL... alternative; have them set global var, that pid loop can just read, faster as we only do it once
float  run_SW1(char* sData, int id)
{
    unsigned char cVal = readSwitch(1);
    delay_ms(1);
    cVal = readSwitch(1);  //need to double read it, to satisfy debounce code

    if(cVal == 1)
    {
        sprintf(sData, "%d,1,", id);
    }
    else
    {
        sprintf(sData, "%d,0,", id);
    }

    return ((float)cVal);
}
float  run_SW2(char* sData, int id)
{
    unsigned char cVal = readSwitch(2);
    delay_ms(1);
    cVal = readSwitch(2);  //need to double read it, to satisfy debounce code

    if(cVal == 1)
    {
        sprintf(sData, "%d,1,", id);
    }
    else
    {
        sprintf(sData, "%d,0,", id);
    }

    return ((float)cVal);
}
//LED Output typical
float run_LV_OUT1(char* sData, int id, int outVal)
{
    int relayNum = 1;

    BOOL ok = setRelay(relayNum, outVal);

    if(VERBOSE)
    {
        if(outVal == 0)
        {
            sprintf(sData, "%d,0,", id);
        }
        else
        {
            sprintf(sData, "%d,1,", id);
        }

    }

    return((float)outVal);
}
//duty 0-99
float run_PWM1(char* sData, int id, int duty)
{
    int pwmNum = 1;

    BOOL ok = setPWM_Output(pwmNum, pwm_reset_timer_val, duty);

    if(VERBOSE)
    {
        sprintf(sData, "%d,%d,", id, duty);
    }


    return((float)duty/100.0);
}
//duty 0-99
float run_PWM2(char* sData, int id, int duty)
{
    int pwmNum = 2;

    BOOL ok = setPWM_Output(pwmNum, pwm_reset_timer_val, duty);

    if(VERBOSE)
    {
        sprintf(sData, "%d,%d,", id, duty);
    }

    return((float)duty/100.0);
}

//using analog right now; may switch later to serial
//currently using RB15/AN9
float run_US1(char* sData, int id)
{
    float fVal;

    fVal = run_ADC(ADC1_Chan, sData, id);

    //run_ADC writes data to string
   //sprintf(sData, "%d,%.3f,", id, fVal);


    return fVal;
}
float run_US2(char* sData, int id)
{
    float fVal = 0.0;

    fVal = run_ADC(ADC2_Chan, sData, id);
    //run_ADC writes data to string
    //sprintf(sData, "%d,%.3f,", id, fVal);

    return fVal;
}
//Get yaw (assume nearly level craft)
float run_HEADING(char* sData, int id)
{
  float heading;
  short int magX, magY, magZ;

    //get compass reading
    //FOR flat mounted box (long axis horizontal, parallel to horizon)
    //heading = get_mag3110_compass(&magX, &magY, &magZ);
    //FOR Vert mounted box (long axis horizontal, parallel to horizon)
    heading = get_mag3110_compassYZ(&magX, &magY, &magZ);

    sprintf(sData, "%d,%.1f,", id, heading);

    return heading;
}
float run_ANGLE_X(char* sData, int id)
{
  float X_AXI, Y_AXI, Z_AXI;

  //get Tilt reading
    get_mpu9150_accel(&X_AXI,&Y_AXI,&Z_AXI);

    sprintf(sData, "%d,%.1f,", id, X_AXI);

    return X_AXI;
}
float run_ANGLE_Y(char* sData, int id)
{
  float fVal=0.0;

  //get Tilt reading
    fVal = get_mpu9150_angle();  //takes 64,100 tics => 1.6ms

    sprintf(sData, "%d,%.1f,", id, fVal);

    return fVal;
}
float run_ANGLE_Z(char* sData, int id)
{
  float fVal=0.0;

  //get Tilt reading
    fVal = get_mpu9150_angle();  //takes 64,100 tics => 1.6ms

    sprintf(sData, "%d,%.1f,", id, fVal);

    return fVal;
}

float run_ANGLE_XYZ(char* sData, int id)
{
  float X_AXI, Y_AXI, Z_AXI;

  //get Tilt reading
    get_mpu9150_accel(&X_AXI,&Y_AXI,&Z_AXI);

    sprintf(sData, "%d,%.1f,,%.1f,,%.1f,", id, X_AXI, Y_AXI, Z_AXI);

    return X_AXI;
}
//GET Roll (for vert box mounted with long axis horizontal to horizon)
float run_TILT_XY(char* sData, int id)
{
  float fVal=0.0;

  //get Tilt reading
    fVal = get_mpu9150_Tilt_XY();  //takes 64,100 tics => 1.6ms

    sprintf(sData, "%d,%.1f,", id, fVal);

    return fVal;
}

//Get Pitch (for vert box mounted with long axis horizontal to horizon)
float run_TILT_XZ(char* sData, int id)
{
  float fVal=0.0;

  //get Tilt reading
//    val = get_mpu9150_angle2();  //takes 64,100 tics => 1.6ms
    fVal = get_mpu9150_Tilt_XZ();  //takes 64,100 tics => 1.6ms

    sprintf(sData, "%d,%.1f,", id, fVal);

    return fVal;
}
float run_TILT_YZ(char* sData, int id)
{
  float fVal=0.0;

  //get Tilt reading
//    val = get_mpu9150_angle2();  //takes 64,100 tics => 1.6ms
    fVal = get_mpu9150_Tilt_YZ();  //takes 64,100 tics => 1.6ms

    sprintf(sData, "%d,%.1f,", id, fVal);

    return fVal;
}

float run_BARO(char* sData, int id)
{
  int pressure_mBar=0, heading=-999, chCnt=0;
  float val=0.0;
  UINT8 tempBUF[80];
  char* pTmpBuf = (char*) &tempBUF[0];
  int temp_degC=20, elev_ft=3010;


    //get pressure (mBar)
    get_LPS331_pressure(&pressure_mBar);
    //convert to sea Level (mBar)
    elev_ft = 3010;
    temp_degC = 20;
    pressure_mBar = convert_pres_to_seaLevel((int) pressure_mBar, elev_ft, temp_degC);

    sprintf(sData, "%d,%d", id, pressure_mBar);

    return((float)pressure_mBar);
 }

//RUN Precision ADC using MCP3553
float run_PADC(int adcNum, char* sData, int id)
{
    int i;
    unsigned int data, dataSum;
    //int numPreReadings = 2;
    //int numReadings = 16;

    float adcVal;
    int chanNum;
    long numTics = 0, ch0_data[14], ch0_data_sum, lt;
    int badReadCnt = 0;
    unsigned int notDataReady;
    unsigned int num_ms;

    chanNum = adcNum;



//mcp3553 code works
    stop_mcp3553(adcNum);
    ch0_data_sum = 0;
    //while(1)
    //{
        //throw away data
        ch0_data_sum = 0;
        badReadCnt = 0;

//CAN'T WORK AS delay_ms(x) also resets core timer!
//reset_Time();

        //for(i=0;i<9;i++)
        for(i=0;i<1;i++)
        {
            start_mcp3553(adcNum);

            delay_ms(1);
            notDataReady = 1;
            do
            {
                notDataReady = spi1_check_mcp3553();
                delay_ms(10); //need a small delay here
            }while(notDataReady>0);

            //take SPI readings and process data
            lt = spi1_process_mcp3553();

            if(lt > 0)
            {
                ch0_data[i] = lt;
                if(i>=0)  //throw out 1st 4
                {
                    ch0_data_sum += lt;
                }
            }
            else
            {
                //don't do this as it could put it into an infinite loop
                //i--;
               // badReadCnt++;
            }

            //continuous mode doesn't seem to work... so do this here; cost is we take a few extra readings in each series
            stop_mcp3553(adcNum);

            //delay_ms(10);
        }

        //need this line if taking more than 1 reading
        //ch0_data_sum = ch0_data_sum>>3;
//num_ms = get_Time_ms();

        //ch0_data_sum = 0;


    //}

    adcVal = (float)ch0_data_sum/1000.0;

    if(adcNum==1)
    {
        adcVal = adcVal * PADC1_SLOPE + PADC1_INTERCEPT;
    }
    else if(adcNum==2)
    {
        adcVal = adcVal * PADC2_SLOPE + PADC2_INTERCEPT;
    }
    else
    {
        adcVal = adcVal * 0.0155;
    }

    data = (int)(1000.0*adcVal);


    sprintf(sData, "%d,%d,", id, data);


    return(adcVal);
}


//this is weird in that adc is composed of
// 1) setting chan, then triggering adc
// 2) later (next call, reading adc
//
//implemented: converts last set ch (nextchanNum)
// then sets nextChan and triggers reading
// so assumes chan already set and has already been triggered last cycle
float run_ADC(int adcNum, char* sData, int id)
{
    int i;
    unsigned int data, dataSum;
    int numPreReadings = 2;
    int numReadings = 16;
    float adcVal;
    int chanNum;


    chanNum = adcNum;

    //setup for next reading
    set_ADC_channel(chanNum);
    delay_us(4);


    for(i=0;i<numPreReadings; i++)
    {
        trigger_ADC();
        delay_us(4);
        read_preTriggered_ADC(&data);
    }

    
    dataSum=0;
    for(i=0;i<numReadings; i++)
    {
        trigger_ADC();
        delay_us(4);
        read_preTriggered_ADC(&data);
        dataSum += data;
    }

    data = dataSum >> 4;

    adcVal = (float)data;


    sprintf(sData, "%d,%d,", id, data);

    
    return(adcVal);
}

long run_Period_SWx(int swNum, char* sData, int id)
{
    float freq = 0.0;
    unsigned int period;

    if(swNum == 1)
        period = period1;
    else if(swNum == 2)
        period = period2;
    else
        period = period1;

    freq = (float)1515.0 / (float)period;
    //3000 IS EQUAL TO TIC RATE FOR ISR
    //IF IT'S NOT GIVING CORRECT ANSWER; CHECK THAT THE Prescalar is set correctly

    sprintf(sData, "%d,%f,", id, freq);
}

//this just reports back last toggle period (goes back to 1 before if roll-over)
//this is enabled by using a high frequency call to readSwitch(x)
//where readSwitch(x) captures TMR4 data at each positive edge transition
long old_run_Period_SWx(int swNum, char* sData, int id)
{
    float val = 0.0;
    long num_tics = 0;
    long num_tics0 = 0;
    long num_tics1 = 0;
    long num_tics2 = 0;
    long tic_1 = 0;

    if(swNum == 1)
    {
        tic_1 = TMR4;
        TMR4 = 0;
                    //delay_ms(1);
                    //tic_2 = TMR4;
        total_tics1 += tic_1;

        if(toggle1_cnt > 0)
        {

            if(toggle1_cnt > 4)
            {
                val = (float)total_tics1 / (100.0 * (float)toggle1_cnt);
                
                toggle1_cnt = 0;
                total_tics1 = 0;
            }
        }
        else
            val = -1.0;



        num_tics0 = timer_sw1_tic_cnt0 - timer_sw1_tic_cnt1;
        num_tics1 = timer_sw1_tic_cnt1 - timer_sw1_tic_cnt2;
        num_tics2 = timer_sw1_tic_cnt2 - timer_sw1_tic_cnt3;

        num_tics = num_tics0;
        
        if(num_tics < 0)
        {
            num_tics = timer_sw1_tic_cnt1 - timer_sw1_tic_cnt2;
        }
        if(num_tics < 0)
        {
            num_tics = timer_sw1_tic_cnt2 - timer_sw1_tic_cnt3;
        }

        //sprintf(sData, "%d,%d,", id, toggle1_cnt);
        if(val > 0.0)
            sprintf(sData, "%d,%f,", id, val);
        else
            sprintf(sData, "%d,,", id);


    }
    else if(swNum == 2)
    {
        num_tics = timer_sw2_tic_cnt0 - timer_sw2_tic_cnt1;

        if(num_tics < 0)
        {
            num_tics = timer_sw2_tic_cnt1 - timer_sw2_tic_cnt2;
        }
        if(num_tics < 0)
        {
            num_tics = timer_sw2_tic_cnt2 - timer_sw2_tic_cnt3;
        }

        //sprintf(sData, "%d,%d,", id, toggle2_cnt);
        sprintf(sData, "%d,%d,", id, num_tics);

    }

    return num_tics;
}


//run program1 (using timing to drive solenoid valve at pre-stored DUTY_CYCLEx values
void run_PROGRAM1(char* sData, int id)
{
    int reset = 0;
    unsigned int ms_cnt = 0;

    //reset switch debounce
    

    unsigned char cVal = readSwitch(1);
    //cVal = readSwitch(1);
    //cVal = readSwitch(2);
    //cVal = readSwitch(2);

    
    sprintf((char*)pTxBuf, "Start program_1\r\n");
    SendDataBufferUART1(pTxBuf, strlen(pTxBuf));

    //initPWM_Output(int pwmNum, int pwmFreq);
    reset = initPWM_Output(1, 100);  //get reset value (timer/clk max val)

    //setPWM_SolidStateRelay_Output(1, reset, duty_cycle1);

    setPWM_Output(1, reset, duty_cycle1);
    //do timing loop1
    reset_Time();
    do
    {
        ms_cnt = get_Time_ms();
    }while(ms_cnt < duration1);

    reset_Time();

    //setPWM_SolidStateRelay_Output(1, reset, duty_cycle2);
    setPWM_Output(1, reset, duty_cycle2);
    //do timing loop1
    do
    {
        ms_cnt = get_Time_ms();
    }while(ms_cnt < duration2);

    //setPWM_SolidStateRelay_Output(1, reset, 0);
    setPWM_Output(1, reset, 1+5);

    sprintf((char*)pTxBuf, "Stop program_1\r\n");

  }
        //switchNum = 1;
        //swVal = readSwitch(switchNum);


void turn_on_solenoid2()
{
        //mPORTBClearBits(BIT_2);
        mPORTBSetBits(BIT_2);
}

void turn_off_solenoid2()
{
        mPORTBClearBits(BIT_2);
        //mPORTBSetBits(BIT_2);

}

void turn_on_solenoid34()
{
        mPORTBSetBits(BIT_4);  //sol. 3

        mPORTASetBits(BIT_4);  //sol. 4
}

void turn_off_solenoid34()
{
        mPORTBClearBits(BIT_4);

        mPORTAClearBits(BIT_4);
}
//uses switches to tell when to stop program;
//also uses duration1 and duration2 as timeouts case switches don't work in time
void run_PROGRAM2(char* sData, int id)
{
    int reset = 0;
    unsigned int ms_cnt = 0;

    unsigned char cVal = readSwitch(1);
    cVal = readSwitch(1);  //must read multiple times to get past debounce code
    cVal = readSwitch(1);
    cVal = readSwitch(2);  //must read multiple times to get past debounce code
    cVal = readSwitch(2);


    //turn on solenoid 2, to allow flow from pwm valve
    turn_on_solenoid2();

    sprintf((char*)pTxBuf, "Start program_2\r\n");
    SendDataBufferUART1(pTxBuf, strlen(pTxBuf));
    delay_milliSecs(100);

    if(cVal > 0)
    {
        sprintf((char*)pTxBuf, "Switch 1 already ON, abort\r\n");
        SendDataBufferUART1(pTxBuf, strlen(pTxBuf));
        delay_milliSecs(100);
        return;
    }


    //initPWM_Output(int pwmNum, int pwmFreq);
    reset = initPWM_Output(1, 100);  //get reset value (timer/clk max val)

//    setPWM_Output(1, reset, duty_cycle1);
    setPWM_SolidStateRelay_Output(1, reset, duty_cycle1);

    //do timing loop1
    reset_Time();
    ms_cnt = get_Time_ms();
    do
    {
        cVal = readSwitch(1);
        if(cVal > 0)
        {
            sprintf((char*)pTxBuf, "Switch 1 triggered\r\n");
            SendDataBufferUART1(pTxBuf, strlen(pTxBuf));
            delay_milliSecs(100);
            break;
        }

        cVal = readSwitch(2);
        if(cVal > 0)
        {
            sprintf((char*)pTxBuf, "Switch 2 triggered before switch 1!\r\n");
            SendDataBufferUART1(pTxBuf, strlen(pTxBuf));
            delay_milliSecs(100);
            break;
        }

        ms_cnt = get_Time_ms();  //timeOut break;
        delay_milliSecs(10);
    }while(ms_cnt < timeOutDuration_ms);

    reset_Time();
    ms_cnt = get_Time_ms();
    
//    setPWM_Output(1, reset, duty_cycle2);
    setPWM_SolidStateRelay_Output(1, reset, duty_cycle2);

    //do timing loop1
    do
    {
        cVal = readSwitch(2);
        if(cVal > 0)
        {
            sprintf((char*)pTxBuf, "Switch 2 triggered\r\n");
            SendDataBufferUART1(pTxBuf, strlen(pTxBuf));
            delay_milliSecs(100);
            break;
        }
        ms_cnt = get_Time_ms();
        delay_milliSecs(10);
    }while(ms_cnt < timeOutDuration_ms);

    //setPWM_Output(1, reset, 5+5);  //for scope testing; makes it more responsive so you can tell when it turns off
    //setPWM_Output(1, reset, 1+5);
setPWM_SolidStateRelay_Output(1, reset, 100);

    turn_off_solenoid2();  //turn off by-pass valve

    sprintf((char*)pTxBuf, "Stop program_2\r\n");
    delay_milliSecs(100);

//where we should turn on solenoids 3&4 (however where/how to turn off after measurement?)
    //turn on solenoid 34, to allow flow from pwm valve
//    turn_on_solenoid34();

  }



//won't use this, just use ptrs
void copy_op(struct OP *destOP, struct OP srcOP)
{
    (*destOP).ParmFreq = srcOP.ParmFreq;
    (*destOP).ParmID = srcOP.ParmID;
    (*destOP).ParmType = srcOP.ParmType;
    (*destOP).ParmTypeNum = srcOP.ParmTypeNum;
    (*destOP).RValType = srcOP.RValType;
    (*destOP).cur_fVal = srcOP.cur_fVal;
    (*destOP).cur_sw_state = srcOP.cur_sw_state;

}


struct OP * find_opId(enum OP_TYPES opType, int *parm_id)
{
     int i, t_id;
     enum OP_TYPES t_oT;
     struct OP *t_op_ptr;

    for(i=0;i<innerLoop_setupCntr; i++)
    {
        t_oT = innerLoop_ops[i].ParmTypeNum;

        if(t_oT == opType)
        {
            *parm_id = innerLoop_ops[i].ParmID;
            t_op_ptr = &innerLoop_ops[i];
            return t_op_ptr;
        }
    }

    for(i=0;i<middleLoop_setupCntr; i++)
    {
        t_oT = middleLoop_ops[i].ParmTypeNum;

        if(t_oT == opType)
        {
            *parm_id = middleLoop_ops[i].ParmID;
            t_op_ptr = &middleLoop_ops[i];
            return t_op_ptr;
        }
    }

    for(i=0;i<outerLoop_setupCntr; i++)
    {
        t_oT = outerLoop_ops[i].ParmTypeNum;

        if(t_oT == opType)
        {
            *parm_id = outerLoop_ops[i].ParmID;
            t_op_ptr = &outerLoop_ops[i];
            return t_op_ptr;
        }
    }

}


//given id, returns parmTypeNum
int find_typeNum(int id)
{
    int i, t_id, parm_type_num=-1;


    for(i=0;i<innerLoop_setupCntr; i++)
    {
        t_id = innerLoop_ops[i].ParmID;

        if(t_id == id)
        {
            parm_type_num = innerLoop_ops[i].ParmTypeNum;
            //op = &innerLoop_ops[i];
            return parm_type_num;
        }
    }

    for(i=0;i<middleLoop_setupCntr; i++)
    {
        t_id = middleLoop_ops[i].ParmID;

        if(t_id == id)
        {
            parm_type_num = middleLoop_ops[i].ParmTypeNum;
            //op = &innerLoop_ops[i];
            return parm_type_num;
        }
    }

    for(i=0;i<outerLoop_setupCntr; i++)
    {
        t_id = outerLoop_ops[i].ParmID;

        if(t_id == id)
        {
            parm_type_num = outerLoop_ops[i].ParmTypeNum;
            //op = &innerLoop_ops[i];
            return parm_type_num;
        }
    }

}


void initLogger()
{
  BOOL ok = FALSE;
  char cVal = 0x00;


    //init sw1 and sw2
    readSwitch(1);
    readSwitch(2);

    // enable multi-vector interrupts
//    INTEnableSystemMultiVectoredInt();

        //IEC0bits.T4IE = 0;  //disable TMR4 interrupt before accessing variable that the ISR modifies (unless it's only a single byte
        //temp1 = sw1_period;
        //temp2 = sw2_period;
    //IEC0bits.T4IE = 1;  //enable TMR4 interrupt


    //setup pressure sensor
    init_LPS331();
    cVal = get_LPS331_who_am_i();
    //should return okt = 0xBB = 187d
    if(cVal==(char)187)
    {
        Installed_Ops[15] = TRUE;
    }

    //compass mag3110
    init_mag3110();
    cVal = get_mag3110_who_am_i();
    if(cVal==(char)0xC4)
    {
        Installed_Ops[16] = TRUE;
    }


    init_mpu9150();
    cVal = get_mpu9150_who_am_i();
    if(cVal==(char)104)
    {
        Installed_Ops[17] = TRUE;
        Installed_Ops[18] = TRUE;
        Installed_Ops[19] = TRUE;
        Installed_Ops[20] = TRUE;
        Installed_Ops[21] = TRUE;
        Installed_Ops[22] = TRUE;
        Installed_Ops[23] = TRUE;
        Installed_Ops[24] = TRUE;
        Installed_Ops[25] = TRUE;
    }


    //init mcp3553
//    init_mcp3553();  //already init inside spi1_init()

}


BOOL parmTypeValid(int parmType)
{
    BOOL ok = TRUE;

    return ok;
    
    //CHECK IF SENSOR IS VALID (i.e. not already assigned (i.e. too many of one type or device doesn't exist ect)


    ok = get_LPS331_who_am_i();
    //should return okt = 0xBB = 187d
    //if(ok=)
    

    return ok;
}


/*
enum OP_TYPES {SW1, SW2, LV_OUT1, PWM1, PWM2,
                US1, US2, PADC1, PADC2, PADC3,
                PADC4, ADC1, ADC2, TC1, TC2,
                BARO, HEADING, TILT_XY, TILT_XZ, TILT_YZ,
                ANGLE_X, ANGLE_Y, ANGLE_Z, GYRO_X, GYRO_Y,
                GYRO_Z};
*/

int get_RValType(enum OP_TYPES opType)
{
    if( (opType == SW1)||(opType == SW2) )
    {
        return(1);
    }
    else if( (opType == LV_OUT1)||(opType == PWM1) || (opType == PWM2))
    {
        return(0);
    }
    else
    {
        return(2);
    }
}

//inputs parmType-freq and sets up dataLogger data-Base that is used in each loop iteration to tell it what to do
BOOL parmSetup(int parmId, int parmType, int parmFreq)
{

    //check if parmType valid
    BOOL ok = parmTypeValid(parmType);

    if((enum OP_TYPES)parmType == PROGRAM1)
    {
        return;  //don't add to list??
    }

    if(parmFreq == inner_loopFreq)
    {
        innerLoop_ops[innerLoop_setupCntr].ParmFreq = parmFreq;
        innerLoop_ops[innerLoop_setupCntr].ParmID = parmId;
        innerLoop_ops[innerLoop_setupCntr].ParmTypeNum = parmType;
        innerLoop_ops[innerLoop_setupCntr].ParmType = (enum OP_TYPES)parmType;
        innerLoop_ops[innerLoop_setupCntr].RValType = get_RValType((enum OP_TYPES)parmType);
        innerLoop_setupCntr++;
    }
    else if(parmFreq == middle_loopFreq)
    {
        middleLoop_ops[middleLoop_setupCntr].ParmFreq = parmFreq;
        middleLoop_ops[middleLoop_setupCntr].ParmID = parmId;
        middleLoop_ops[middleLoop_setupCntr].ParmTypeNum = parmType;
        middleLoop_ops[middleLoop_setupCntr].ParmType = (enum OP_TYPES)parmType;
        middleLoop_ops[middleLoop_setupCntr].RValType = get_RValType((enum OP_TYPES)parmType);

        middleLoop_setupCntr++;
    }
    else if(parmFreq == outer_loopFreq)
    {
        outerLoop_ops[outerLoop_setupCntr].ParmFreq = parmFreq;
        outerLoop_ops[outerLoop_setupCntr].ParmID = parmId;
        outerLoop_ops[outerLoop_setupCntr].ParmTypeNum = parmType;
        outerLoop_ops[outerLoop_setupCntr].ParmType = (enum OP_TYPES)parmType;
        outerLoop_ops[outerLoop_setupCntr].RValType = get_RValType((enum OP_TYPES)parmType);

        //outer_loop_ids[outer_loop_setupCntr] = parmType;
        outerLoop_setupCntr++;
    }

}


//outVal 0-100 (% of dutycycle)

//runOutputOp(pid->outputOp_ptr->ParmType, pid->reset_tmr_tic_cnt, iDuty);
void runOutputOp(enum OP_TYPES opType, int reset, int duty)
{
    int relayNum, pwmNum;
  //LONG CASE STATEMENT HERE (OR CASCADED SERIES OF IF STATEMENTS)

  if(opType == LV_OUT1)
  {
      relayNum = 1;
      if(duty > 0)
          setRelay(relayNum, 0);
      else
          setRelay(relayNum, 1);

  }
  else if(opType == PWM1)
  {
      pwmNum = 1;
      setPWM_Output(pwmNum, reset, duty);
  }
  else if(opType == PWM2)
  {
      pwmNum = 2;
      setPWM_Output(pwmNum, reset, duty);
  }

}

//callingLoop is which loop is calling {initLoop=0, inner=1, middle=2, outer=3}
//chks if correct loop is calling and only exe's for the right 1
//ignores chk if loop=0; i.e. initLoop calling

void exe_pid_loop(struct PID *pid, int callingLoop)
{
    //float input, output, kp, ki, kd, ITerm, lastInput, outMin, outMax, SetPoint;
    float Output, fDuty;
    int iDuty;

    int pFreq = pid->inputOp_ptr->ParmFreq;
    int cLoop, reset;
    enum OP_TYPES opType;


    if(pid->control_state == 1)
    {

        if(pFreq == inner_loopFreq)
        {
            cLoop = 1;
        }
        else if(pFreq == middle_loopFreq)
        {
            cLoop = 2;
        }
        else
        {
            cLoop = 3;
        }


        if( (callingLoop == 0)||(cLoop = callingLoop) )
        {

            //pid1.ParmType
            //float PID_Compute(float Input, float Setpoint, float outMin, float outMax, float kp, float ki, float kd, float *ITerm, float *lastInput);
            //float PID_Compute(float Input, float Setpoint, float outMin, float outMax, float kp, float ki, float kd, float *ITerm, float *lastInput)
            PID_Compute(pid->inputOp_ptr->cur_fVal, pid->setPoint, pid->outMin, pid->outMax, pid->kp, pid->ki, pid->kd, &(pid->ITerm), &(pid->lastInput), &Output);

            //convert output to duty cycle
            if(pid->outMax > 0.0)
            {
                fDuty = Output / pid->outMax;
            }
            else
            {
                return;
            }

            iDuty = (int)(100.0 * fDuty);

            opType = pid->outputOp_ptr->ParmType;
            reset = pid->reset_tmr_tic_cnt;
            
            //runOutputOp(pid->outputOp_ptr->ParmType, pid->reset_tmr_tic_cnt, iDuty);
            runOutputOp(opType, reset, iDuty);
        }
    }

}

//runs at freq of input var
void init_PID_Loop(struct PID *pid)
{
    //find freq of input var
    int pidLoopFreq_ms = (int)(1000.0/((float)(pid->inputOp_ptr->ParmFreq)));
    int control_dir;
    float output=0.0;



    control_dir = pid->control_direction;
    
    PID_SetTunings(pidLoopFreq_ms, &(pid->kp), &(pid->ki), &(pid->kd), control_dir);

    if(pid->control_state)
    {
        pid->inputOp_ptr->cur_fVal = 3.65;
        //PID_Initialize(Input, &lastInput, Output,  &ITerm, outMax, outMin);
        PID_Initialize(pid->inputOp_ptr->cur_fVal, &(pid->lastInput), &output,  &(pid->ITerm), pid->outMax, pid->outMin);



        //need to calc reset rollover tic count
        //need to initialize pwm modules
        //what to do with output?  how do we take output and execute pid control?
        //runOutputOp(int parmTypeNum, int reset, int duty);
        exe_pid_loop(pid, 0);  //set callingLoop parm=0 so it's guaranteed to run
    }
}

void start_PID_Loop(struct PID *pid)
{
    //PID_Initialize(Input, &lastInput, Output,  &ITerm, outMax, outMin);
    PID_Initialize(pid->inputOp_ptr->cur_fVal, &(pid->lastInput), pid->lastInput,  &(pid->ITerm), pid->outMax, pid->outMin);

    //note: passing ptr of sub-element of struct may be &(ptr)->a rather than our &(ptr->a)
    
    exe_pid_loop(pid,0);  //set callingLoop parm=0 so it's guaranteed to run
    
    pid->control_state = 1;
}


void stop_PID_Loop(struct PID *pid)
{
    pid->control_state = 0;
}

//rxd string example (! is start/stop pid, 1st n(1-2) is which pwm we're requesting change for; 2nd n(0,1) is on/off (2/1)
//!,2,2  ;start pwm2
//!,1,2  ;start pwm1
//!,2,1  ;stop pwm2
void startStop_PID_Loop(unsigned char *sConfig)
{
    unsigned char* sPtr = sConfig;
    int* ptrParms;
    int numParms = 0;  //spec out max # parms
    int pidNum;
    int pidState;
    int startStopParms[2];
    int maxNumParms = 2;
    int maxChars = 2;

    sPtr++; //skip '!'
    sPtr++; //skip ','

    //get numParms and parms; returns parms (doesn't include #parms parm)
    numParms = getIntegerParam_in_csv(sPtr, maxChars, &startStopParms[0], maxNumParms);

    if(numParms >= 2)
    {
        ptrParms = &startStopParms[0];

        pidNum = *ptrParms++;
        pidState = *ptrParms - 1;

        if(pidNum == 1)
        {
            pid1.control_state = pidState;
        }
        else if(pidNum == 2)
        {
            pid2.control_state = pidState;
        }
    }

}

//rxd string example
//@,pid#,kd,kp,ki,inputVar_opType,outputVar_opType,controlDir,controlState,outMax,outMin  these are all floats....
//assumes a unique opType (i.e. only 1 unique element)
void config_PID_Loop(unsigned char *sConfig, int tics_per_ms)
{
    unsigned char* sPtr = sConfig;
    int maxChars = 10;

    float* ptrParms;
    int numParms = 0;  //spec out max # parms
    int parmIndx = 0;

    float kp, kd, ki, outMax, outMin, setPoint;
    int pid_num, pid_control_inputVar_opType, pid_control_outputVar_opType, control_dir, control_state;

    struct OP *inOp_ptr, *outOp_ptr;
    struct OP op;

    sPtr++; //skip @
    sPtr++; //skip ','



    //get numParms and parms; returns parms (doesn't include #parms parm)
    //numParms = getIntegerParam_in_csv(sPtr, maxChars, &configParms[0], maxNumParms);
    numParms = getFloatParam_in_csv(sPtr, maxChars, &configPID_Parms[0], maxNumParms);


    ptrParms = &configPID_Parms[0];


    if(numParms == 11)
    {
        pid_num = configPID_Parms[parmIndx++];
        
        kp = (configPID_Parms[parmIndx++]);  
        kd = (configPID_Parms[parmIndx++]);
        ki = (configPID_Parms[parmIndx++]);


        pid_control_inputVar_opType = (int)configPID_Parms[parmIndx++];
        pid_control_outputVar_opType = (int)configPID_Parms[parmIndx++];

        control_dir = (int)configPID_Parms[parmIndx++];

        control_state = (int)configPID_Parms[parmIndx++];  //on(1) or off(0)

        outMax = configPID_Parms[parmIndx++];
        outMin = configPID_Parms[parmIndx++];

        setPoint = configPID_Parms[parmIndx++];

        if(pid_num == 1)
        {
            inOp_ptr = find_opId((enum OP_TYPES)pid_control_inputVar_opType, &pid1.inputOp_id);  //get pointer to provided Op ID
            outOp_ptr = find_opId((enum OP_TYPES)pid_control_outputVar_opType, &pid1.outputOp_id);  //get pointer to provided Op ID

            pid1.inputOp_ptr = inOp_ptr;
            pid1.outputOp_ptr = outOp_ptr;

            pid1.input_opType = (enum OP_TYPES)pid_control_inputVar_opType;
            pid1.output_opType = (enum OP_TYPES)pid_control_outputVar_opType;

            pid1.kd = kd;
            pid1.ki = ki;
            pid1.kp = kp;

            pid1.control_direction = control_dir;
            pid1.control_state = control_state;

            pid1.outMax = outMax;
            pid1.outMin = outMin;

            pid1.setPoint = setPoint;

            //adjusts kp, ki, kd for loop freq &
            //chks if control_state is active & if so, init vars & runs pid loop
            init_PID_Loop(&pid1);
        }
        else if(pid_num == 2)
        {
            //find_opId((enum OP_TYPES)pid_control_inputVar_opType, &inOp);  //get pointer to provided Op ID
            //find_opId((enum OP_TYPES)pid_control_outputVar_opType, &outOp);  //get pointer to provided Op ID
            inOp_ptr = find_opId((enum OP_TYPES)pid_control_inputVar_opType, &pid2.inputOp_id);  //get pointer to provided Op ID
            outOp_ptr = pid2.outputOp_id = find_opId((enum OP_TYPES)pid_control_outputVar_opType, &pid2.outputOp_id);  //get pointer to provided Op ID

            //op = *inOp_ptr;

            pid2.inputOp_ptr = inOp_ptr;
            pid2.outputOp_ptr = outOp_ptr;

            //op = *pid2.inputOp_ptr;

            
            
            pid2.input_opType = (enum OP_TYPES)pid_control_inputVar_opType;
            pid2.output_opType = (enum OP_TYPES)pid_control_outputVar_opType;

            pid2.kd = kd;
            pid2.ki = ki;
            pid2.kp = kp;

            pid2.control_direction = control_dir;
            pid2.control_state = control_state;

            pid2.outMax = outMax;
            pid2.outMin = outMin;

            pid2.setPoint = setPoint;

            //adjusts kp, ki, kd for loop freq &
            //chks if control_state is active & if so, init vars & runs pid loop
            init_PID_Loop(&pid2);
        }


    }
    else
    {
        sprintf((char*)sPtr, "Wrong # of parms in config PID Request\r\n");
        SendDataBufferUART1(sPtr, strlen(sPtr));

    }

}





//accepts a string that should hold data-logger config (note: freq must be {1, 10, 100Hz})
//$,4,1,1000,2,1000,5,1000,5,1000
BOOL configDataLogger(unsigned char *sConfig, int tics_per_ms, int *tics_per_loop)
{
    unsigned char* sPtr = sConfig;
    int maxChars = 6;
    int numParms = 0, numParmsRxd;  //spec out max # parms
    BOOL ok = FALSE;
    int num_ms_per_loop = 100;
    int* ptrParms;
    int parmType, parmFreq;
    int maxFreq = 0;
    int parmCnt = 0;
    int parmIndx = 0;

    //ensure if pidConfig doesn't get called that it's in a good state i.e. won't run
    pid1.control_state = 0;
    pid2.control_state = 0;

    //reset loop setupCntr's so we don't just add on to end of them when we re-run this config
    innerLoop_setupCntr = 0;
    middleLoop_setupCntr = 0;
    outerLoop_setupCntr = 0;

    
    sPtr++; //skip $
    sPtr++; //skip ','

    //get numParms and parms; returns parms (doesn't include #parms parm)
    //CAN'T USE 0 FOR TYPEID AS getIntegerParam aborts on '0'
    numParms = getIntegerParam_in_csv(sPtr, maxChars, &configParms[0], maxNumParms);

    ptrParms = &configParms[0];

    if(numParms >= 1)
    {
        //numParmsRxd = *ptrParms++;
        numParmsRxd = configParms[0];
        parmIndx++;
        do
        {
            //parmType = *ptrParms++;
            //parmFreq = *ptrParms++;
            parmType = configParms[parmIndx++];
            parmFreq = configParms[parmIndx++];


            parmCnt+=2;

            

            if(parmType == 0)
                break;

            //special is loop_cnt, where we use parmFreq to set # times through loop
            if((enum OP_TYPES)parmType == LOOP_COUNT)
            {
                loop_cnt = parmFreq;
            }
            else if((enum OP_TYPES)parmType == PROGRAM1)
            {
                PROGRAM1_Enabled = TRUE;
            }
            else if((enum OP_TYPES)parmType == PROGRAM2)
            {
                PROGRAM2_Enabled = TRUE;
            }
            else if((enum OP_TYPES)parmType == DUTY_CYCLE1)
            {
                duty_cycle1 = parmFreq;
            }
            else if((enum OP_TYPES)parmType == DUTY_CYCLE2)
            {
                duty_cycle2 = parmFreq;
            }
            else if((enum OP_TYPES)parmType == DURATION_1_ms)
            {
                duration1 = parmFreq;
            }
            else if((enum OP_TYPES)parmType == DURATION_2_ms)
            {
                duration2 = parmFreq;
            }
            //PROGRAM1_Enabled

            else
            {
                //setup this parm
                ok = parmSetup(parmCnt>>1, parmType, parmFreq);


                if(parmFreq > maxFreq)
                {
                    maxFreq = parmFreq;
                }

                if(ok)
                    dataLogSetup = TRUE;
            }

        }while(parmCnt < numParms);

    }

    //calc tics_per_loop from maxFreq

    //or just use default...
    //need to extract # ms per loop
    *tics_per_loop = tics_per_ms * num_ms_per_loop;

        //need to use parms to set flags

 

    return(ok);
}


//rxd string example (! is start/stop relay, 1st n(1-2) is which pwm we're requesting change for; 2nd n(1,2) is on/off (2/1)
//%,2,2  ;turn on relay 2
//%,1,2  ;turn on relay 1
//%,2,1  ;turn off relay 2
void startStop_Relay(unsigned char *sConfig)
{
    unsigned char* sPtr = sConfig;
    int* ptrParms;
    int numParms = 0;  //spec out max # parms
    int relayNum;
    int relayState;
    int startStopParms[2];
    int maxNumParms = 2;
    int maxChars = 2;

    sPtr++; //skip '!'
    sPtr++; //skip ','

    //get numParms and parms; returns parms (doesn't include #parms parm)
    numParms = getIntegerParam_in_csv(sPtr, maxChars, &startStopParms[0], maxNumParms);

    if(numParms >= 2)
    {
        ptrParms = &startStopParms[0];

        relayNum = *ptrParms++;
        relayState = *ptrParms - 1;

        setRelay(relayNum, relayState);
        
    }

}


//allows PC to turn on/off LEDS, Relays, adjust PWM settings ect
BOOL getPcControlInputDataLogger(char *sData, int id, int parmTypeNum, int outVal)
{
  //LONG CASE STATEMENT HERE (OR CASCADED SERIES OF IF STATEMENTS)

  if(parmTypeNum == 2)
  {
      run_LV_OUT1(sData, id, outVal);
  }
  else if(parmTypeNum == 3)
  {
      run_PWM1(sData, id, outVal);
  }
  else if(parmTypeNum == 4)
  {
      run_PWM2(sData, id, outVal);
  }

    
  return TRUE;
}


/*
enum OP_TYPES {SKIP, SW1, SW2, LV_OUT1, PWM1, PWM2,
                US1, US2, PADC1, PADC2, PADC3,
                PADC4, ADC1, ADC2, TC1, TC2,
                BARO, HEADING, TILT_XY, TILT_XZ, TILT_YZ,
                ANGLE_X, ANGLE_Y, ANGLE_Z, GYRO_X, GYRO_Y,
                GYRO_Z};
 */

//RUNS a single op as requested in parmTypeNum
//outVal contains value we need to send to funcs that set things like relays and PWM outputs
float runDataLogOp(char *sData, int id, int parmTypeNum, enum OP_TYPES optype, int outVal)
{
    float val=0.0;
  //LONG CASE STATEMENT HERE (OR CASCADED SERIES OF IF STATEMENTS)



  //using parmTypeNum makes more sense as they're unique addresses
  id = parmTypeNum;



  if(parmTypeNum == 1)
  {
      val = run_SW1(sData, id);
  }
  else if(parmTypeNum == 2)
  {
      val = run_SW2(sData, id);
  }
  else if(parmTypeNum == 3)
  {
      val = run_LV_OUT1(sData, id, outVal);
  }
  else if(parmTypeNum == 4)
  {
      val = run_PWM1(sData, id, outVal);
  }
  else if(parmTypeNum == 5)
  {
      val = run_PWM2(sData, id, outVal);
  }
  else if(parmTypeNum == 6)
  {
      val = run_US1(sData, id);
  }
  else if(parmTypeNum == 7)
  {
      val = run_US2(sData, id);
  }
    /*
  else if(parmTypeNum == 8)
  {
      val = run_PADC1(sData, id);
  }
  else if(parmTypeNum == 9)
  {
      val = run_PADC2(sData, id);
  }
  else if(parmTypeNum == 10)
  {
      val = run_PADC3(sData, id);
  }
  else if(parmTypeNum == 11)
  {
      val = run_PADC4(sData, id);
  }
    */
  else if(parmTypeNum == 12)
  {
      val = run_ADC((int)ADC1_Chan, sData, id);
  }
  else if(parmTypeNum == 13)
  {
      val = run_ADC((int)ADC2_Chan, sData, id);
  }
  //else if(parmTypeNum == 14)
  //{
  //    val = run_TC1(sData, id);
  //}
  //else if(parmTypeNum == 15)
  //{
  //    val = run_TC2(sData, id);
  //}
  else if(parmTypeNum == 16)
  {
      val = run_BARO(sData, id);
  }
  else if(parmTypeNum == 17)
  {
      val = run_HEADING(sData, id);
  }
  else if(parmTypeNum == 18)
  {
      val = run_TILT_XY(sData, id);
  }
  else if(parmTypeNum == 19)
  {
      val = run_TILT_XZ(sData, id);
  }
  else if(parmTypeNum == 20)
  {
      val = run_TILT_YZ(sData, id);
  }
  else if(parmTypeNum == 21)
  {
      val = run_ANGLE_X(sData, id);
  }
  else if(parmTypeNum == 22)
  {
      val = run_ANGLE_Y(sData, id);
  }
  else if(parmTypeNum == 23)
  {
      val = run_ANGLE_Z(sData, id);
  }
  else if((enum OP_TYPES)optype == PADC1)  //precision adc1
  {
      val = run_PADC((int)1, sData, id);
  }
  else if((enum OP_TYPES)optype == PADC2)  //precision adc1
  {
      val = run_PADC((int)2, sData, id);
  }
  else if((enum OP_TYPES)optype == Period_SW1)  //precision adc1
  {
      //readSwitch(1);
      //readSwitch(2);
      
      val = (int)run_Period_SWx((int)1, sData, id);
  }
  else if((enum OP_TYPES)optype == Period_SW2)  //precision adc1
  {
      val = (int)run_Period_SWx((int)2, sData, id);
  }


    return val;
}



//assumes this gets called repeatedly; i.e. this only performs 1 set of data, as spec'd earlier by call to config
//adds each data set to passed in string
//data_throttling will occur outside this fun; throttling should occur at max innerloop rate
//INPUTS ONLY, doesn't set anything
void runDataLogIteration(char *sData)
{
    int i, id, parm_type_num;
    int run_middleLoop = loopCount % middle_loopFreq;
    int run_outerLoop = loopCount % inner_loopFreq;
    enum OP_TYPES optype;

    char* sPtr = sData;
    char tBuf[80];
    char* sBuf;
    float rVal;
    int callingLoop;

    int val = 0;


    if(PROGRAM1_Enabled == TRUE)
    {
        run_PROGRAM1(sData, id);
        //TURN OFF PROGRAM1
        PROGRAM1_Enabled = FALSE;
    }
    else if(PROGRAM2_Enabled == TRUE)
    {
        run_PROGRAM2(sData, id);
        //TURN OFF PROGRAM2
        PROGRAM2_Enabled = FALSE;
    }

    if(loop_cnt == 0)
        return;   //stop




    sprintf(sPtr, "$,");  //setup header (assume everytime we're going to send something; can test string length just before sending to see if we need to)
            //sprintf((char*)sData, "$,%.1f,%d,%d\r\n",val, heading, pressure_mBar);

    //here-after need to cat string, not sprintf, so sprintf to tBuf, then cat onto sPtr; example below on how to add id to string
    //sprintf(tBuf, "%d,", id);
    //strcat(sPtr, tBuf);


    loopCount++;

    //always run inner loop
    for(i=0;i<innerLoop_setupCntr; i++)
    {
        readSwitch(1);
        readSwitch(2);

        id = innerLoop_ops[i].ParmID;
        parm_type_num = innerLoop_ops[i].ParmTypeNum;
        optype = innerLoop_ops[i].ParmType;

        sBuf = &tBuf[0];
        rVal = runDataLogOp(sBuf, id, parm_type_num, optype, 0);

        //save reading for use by pid controller
        if(innerLoop_ops[i].RValType == 1) //int val (switch)
        {
            innerLoop_ops[i].cur_sw_state = (int)rVal;
        }
        else if(innerLoop_ops[i].RValType == 2) //float val (sensor)
        {
            innerLoop_ops[i].cur_fVal = rVal;        

        }
    
        //now cat returned string onto our full data string
        strcat(sPtr, tBuf);
    }

    //now that innerLoop vars have all been read, call pid's (will only run if they're innerLoop pid's)
    callingLoop = 1;
    if(pid1.control_state == 1)
    {
        exe_pid_loop(&pid1, callingLoop);
    }
    if(pid2.control_state == 1)
    {
        exe_pid_loop(&pid2, callingLoop);
    }


    if(run_middleLoop == 0)
    {
        for(i=0;i<middleLoop_setupCntr; i++)
        {
            readSwitch(1);
            readSwitch(2);

            id = middleLoop_ops[i].ParmID;
            parm_type_num = middleLoop_ops[i].ParmTypeNum;
            optype = middleLoop_ops[i].ParmType;

            sBuf = &tBuf[0];
            runDataLogOp(sBuf, id, parm_type_num, optype, 0);

            //now cat returned string onto our full data string
            strcat(sPtr, tBuf);
        }

        //now that middleLoop vars have all been read, call pid's (will only run if they're innerLoop pid's)
        callingLoop = 2;
        if(pid1.control_state == 1)
        {
            exe_pid_loop(&pid1, callingLoop);
        }
        if(pid2.control_state == 1)
        {
            exe_pid_loop(&pid2, callingLoop);
        }

    }

    if(run_outerLoop == 0)
    {
        for(i=0;i<outerLoop_setupCntr; i++)
        {
            readSwitch(1);
            readSwitch(2);

            id = outerLoop_ops[i].ParmID;
            parm_type_num = outerLoop_ops[i].ParmTypeNum;
            optype = outerLoop_ops[i].ParmType;

            sBuf = &tBuf[0];
            runDataLogOp(sBuf, id, parm_type_num, optype, 0);

            //now cat returned string onto our full data string
            strcat(sPtr, tBuf);
        }

        //now that outerLoop vars have all been read, call pid's (will only run if they're innerLoop pid's)
        callingLoop = 3;
        if(pid1.control_state == 1)
        {
            exe_pid_loop(&pid1, callingLoop);
        }
        if(pid2.control_state == 1)
        {
            exe_pid_loop(&pid2, callingLoop);
        }

        //reset loop count once every time through outerloop
        loopCount = 0;
    }

    loop_cnt--;  //reduce main loop_cnt (that keeps track of how many times we collect data before quiting

    sprintf((char*)tBuf, "\n");
    strcat(sPtr, tBuf);
}