//I2C
#include <plib.h>


#define I2C_CLOCK_FREQ 100000
#define	SYSTEM_CLOCK 			(40000000ul)

#define SYSCLK	(40000000)
#define PBCLK  (SYSCLK)
//#pragma config FPBDIV = DIV_1				//Sets PBCLK to SYSCLK

#define Fsck	50000
#define BRG_VAL 	((PBCLK/2/Fsck)-2)



void i2c_wait(unsigned int cnt)
{
	while(--cnt)
	{
		Nop();
		Nop();
	}
}

/*****************************************************
 * RcvData(unsigned int address)		     *
 *					  	     *
 * Gets a byte of data from I2C slave device at      *
 *  ADDRESS.					     *
 *						     *
 * Returns: Received data			     *
 ****************************************************/
int i2c1_RcvData(unsigned int address) 
{

	StartI2C1();				//Send line start condition
	IdleI2C1();			        //Wait to complete
	MasterWriteI2C1((address ) | 1);	//Write out slave address OR 1 (read command)
	IdleI2C1();				//Wait to complete
	int rcv = MasterReadI2C1();		//Read in a value
	StopI2C1();				//Send line stop condition
	IdleI2C1();				//Wait co complete
	return rcv;				//Return read value
}

int i2c1_Get_Data(unsigned int address, unsigned int reg)
{
   int rcv;


	StartI2C1();	        //Send the Start Bit
	IdleI2C1();		//Wait to complete

	MasterWriteI2C1((address ) | 0);  //Sends the slave address over the I2C line.  This must happen first so the 
                                             //proper slave is selected to receive data.
	IdleI2C1();	        //Wait to complete

	MasterWriteI2C1(reg);  //Sends register # as a data byte over I2C line
	IdleI2C1();		//Wait to complete

	StartI2C1();	        //Send the Start Bit
	IdleI2C1();				//Wait to complete

	MasterWriteI2C1((address ) | 1);  //Sends the slave address over the I2C line.  This must happen first so the 
	IdleI2C1();	        //Wait to complete
	rcv = MasterReadI2C1();		//Read in a value
	StopI2C1();				//Send line stop condition
	IdleI2C1();				//Wait co complete
	return rcv;				//Return read value


}



int i2c1_Send_DataReg(unsigned int address, unsigned int reg, unsigned int data)
{
   int rcv;


	StartI2C1();	        //Send the Start Bit
	IdleI2C1();		//Wait to complete

	MasterWriteI2C1((address ) | 0);  //Sends the slave address over the I2C line.  This must happen first so the 
                                             //proper slave is selected to receive data.
	IdleI2C1();	        //Wait to complete

	MasterWriteI2C1(reg);  //Sends register # as a data byte over I2C line

	IdleI2C1();		//Wait to complete

	MasterWriteI2C1(data);  //Sends the slave address over the I2C line.  This must happen first so the 

	IdleI2C1();				//Wait co complete
	StopI2C1();				//Send line stop condition
	IdleI2C1();				//Wait co complete

	return rcv;				//Return read value


}



/***************************************************
 * SendData(int data, unsigned int address)        *
 *						    *
 * Sends a byte of data (DATA) over the I2C line   *
 *	to I2C address ADDRESS			    *
 *						    *
 * Returns: nothing				    *
 ***************************************************/
void i2c1_SendData (int data, unsigned int address)
{


	StartI2C1();	        //Send the Start Bit
	IdleI2C1();		//Wait to complete

	MasterWriteI2C1((address ) | 0);  //Sends the slave address over the I2C line.  This must happen first so the 
                                             //proper slave is selected to receive data.
	IdleI2C1();	        //Wait to complete

	MasterWriteI2C1(data);  //Sends data byte over I2C line
	IdleI2C1();		//Wait to complete

	StopI2C1();	        //Send the Stop condition
	IdleI2C1();	        //Wait to complete

} //end function


/*
void Delayms( unsigned t)
// This uses Timer 1, can be changed to another timer. Assumes FPB = SYS_FREQ
{
    OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
    while (t--)
    {  // t x 1ms loop
        WriteTimer1(0);
        while (ReadTimer1() < SYS_FREQ/256/1000);
	}
	CloseTimer1();
} // Delayms
*/


//chanNum = i2c channel we're using
//no analog on sda1, scl1 lines & not using sda2
void init_i2c(int chanNum)
{
   long actualClock = 0;
   int group;

//return;

	//setup pins for i2c1
	PORTSetPinsDigitalOut(IOPORT_B, BIT_9);  //I2C1, SDA; no analog, just digital
	PORTSetPinsDigitalOut(IOPORT_B, BIT_8);	 //I2C1, SCL; no analog, just digital



	// Configure Various I2C Options
	if(chanNum == 1)
	{

		I2CEnable(I2C1, FALSE);

		//I2CConfigure(I2C1, I2C_ENABLE_SLAVE_CLOCK_STRETCHING|I2C_ENABLE_HIGH_SPEED);
		I2CConfigure(I2C1, 0);

		// Set Desired Operation Frequency
		actualClock = I2CSetFrequency(I2C1, SYSTEM_CLOCK, I2C_CLOCK_FREQ);


		//  Set the Slave Address
		I2CSetSlaveAddress(I2C1, 0x3A, 0, I2C_USE_7BIT_ADDRESS);
		DBPRINTF("  Slave address = 0x%02X\n", MY_SLAVE_ADDRESS);



		// Enable the module (must do for master as well, or nothing happens when you txd)
		//DBPRINTF("Enabling slave operations...\n");
		I2CEnable(I2C1, TRUE);

	//Enable I2C channel and set the baud rate to BRG_VAL)
	OpenI2C1( I2C_EN, BRG_VAL );

		i2c_wait(10);

		//IdleI2C2();
		if (I2CBusIsIdle(I2C1)==1)
		{
			group = 1;
		}
	}
	else
	{
		I2CConfigure(I2C2, I2C_ENABLE_SLAVE_CLOCK_STRETCHING|I2C_ENABLE_HIGH_SPEED);
		// Set Desired Operation Frequency
		actualClock = I2CSetFrequency(I2C2, SYSTEM_CLOCK, I2C_CLOCK_FREQ);

		// Enable the module
		DBPRINTF("Enabling slave operations...\n");
		I2CEnable(I2C2, TRUE);
	}
	
	/*
	if ( abs(actualClock-I2C_CLOCK_FREQ) > I2C_CLOCK_FREQ/10 )
	{
	    DBPRINTF("  Clock frequency (%d) error exceeds 10\%\r\n", actualClock);
	}
	else
	{
	    DBPRINTF("  Clock frequency = %ld Hz\n", actualClock);
	}
	
	
	//  Set the Slave Address
	I2CSetSlaveAddress(I2C1A, MY_SLAVE_ADDRESS, 0, I2C_USE_7BIT_ADDRESS);
	DBPRINTF("  Slave address = 0x%02X\n", MY_SLAVE_ADDRESS);
	

	

	// Enable the module
	DBPRINTF("Enabling slave operations...\n");
	I2CEnable(I2C1A, TRUE);
		
	*/
}



/*
int chanReady_i2c(int chanNum)
{
   int result, data = 0x00;

return(1);

    if(chanNum == 1)
	{
		if (I2CTransmitterIsReady(I2C1))
		{
			return(1);
		    //result = I2CSendByte(I2C1, data);
		    //if(result != I2C_SUCCESS)
		    //{
		    //    DBPRINTF("Error %d, attempting to send data\n", result);
			//	return(0);
		    //}
		}
		else
		{
			return(0);
		}
	}

	return(1);

}



//returns 1 if successful, 0 if not
int getByte_i2c(int chanNum, int slaveAddress, int *val)
{
   int result = 0xFF;
   int ok = 0;

	if(chanReady_i2c(chanNum))
	{
		if(chanNum == 1)
		{
			I2CStart(I2C1);

			if (I2CTransmitterIsReady(I2C1))
			{
			    result = I2CSendByte(I2C1, slaveAddress);
			    result = I2CSendByte(I2C1, 0x00);
			}
	
	
			I2CStop(I2C1);

			*val = result;

			return(1);  //return 'ok'
		}


	}

	return(ok);
}


//returns 1 if successful, 0 if not
int getByte_fromReg_i2c(int chanNum, int slaveAddress, int regAddress, int *val)
{
   int result = 0xFF;
   int ok = 0;



	if (I2CBusIsIdle(I2C1))
	{
		if(chanNum == 1)
		{




			I2CStart(I2C1);
			IdleI2C2();

			result = I2CSendByte(I2C1, slaveAddress);  //use base address for write
			result = I2CSendByte(I2C1, 0x00);

			//I2CStart(I2C1);  //could be repeaterd start
			I2CRepeatStart(I2C1);

			result = I2CSendByte(I2C1, slaveAddress+1);  //use base address+1 for read
			result = I2CSendByte(I2C1, 0x00);   		 //read data byte

			//result = I2CGetByte(I2C1);

			if (I2CReceivedDataIsAvailable(I2C1))
				{
				    result = I2CGetByte(I2C1);

					*val = result;

					return(1);  //return 'ok'
				}



			if (I2CTransmitterIsReady(I2C1))
			{

			    result = I2CSendByte(I2C1, slaveAddress);  //use base address for write
			    result = I2CSendByte(I2C1, 0x00);

				I2CStart(I2C1);  //could be repeaterd start
				//I2CRepeatStart(I2C1);

			    result = I2CSendByte(I2C1, slaveAddress+1);  //use base address+1 for read
				result = I2CSendByte(I2C1, 0x00);   		 //read data byte

			    //result = I2CGetByte(I2C1);
				
			if (I2CReceivedDataIsAvailable(I2C1))
				{
				    result = I2CGetByte(I2C1);

					*val = result;

					return(1);  //return 'ok'
				}

				I2CAcknowledgeByte(I2C1, TRUE);


				I2CStop(I2C1);


				

			}
	
	

		
		}


	}

	return(ok);
}

//returns 1 if successful, 0 if not
int getWord_i2c(int chanNum, int slaveAddress, int *val)
{

}

//returns 1 if successful, 0 if not
int writeByte_i2c(int chanNum, int slaveAddress, int val)
{
   int result = 0xFF;
   int ok = 0;

	if(chanReady_i2c(chanNum))
	{
		if(chanNum == 1)
		{
			I2CStart(I2C1);

			if (I2CTransmitterIsReady(I2C1))
			{
			    result = I2CSendByte(I2C1, slaveAddress);
			    result = I2CSendByte(I2C1, val);
			}
	
	
			I2CStop(I2C1);

			return(1);  //return 'ok'
		}


	}

	return(ok);
}


//returns 1 if successful, 0 if not
int writeByte2reg_i2c(int chanNum, int slaveAddress, int regAddress, int val)
{
   int result = 0xFF;
   int ok = 0;

	if(chanReady_i2c(chanNum))
	{
		if(chanNum == 1)
		{
			I2CStart(I2C1);

			if (I2CTransmitterIsReady(I2C1))
			{
			    result = I2CSendByte(I2C1, slaveAddress);  //use base address for write
			    result = I2CSendByte(I2C1, regAddress);    //register address
			    result = I2CSendByte(I2C1, val);		   //send data
			}
	
	
			I2CStop(I2C1);

			return(1);  //return 'ok'
		}


	}

	return(ok);
}

*/