#include "i2c.h"
#include "math_utils.h"



//#define mag3110_i2c_addr 0x0E   //0x0E needs to be shifted left 1
#define mag3110_i2c_addr 0x1C    //0x1C  address after shift
                                //1st bit is R/W with R==1 and W==0

#define mag3110_W_ADDRESS   mag3110_i2c_addr             //  ( BIT0 ==0 for writes)
#define mag3110_R_ADDRESS   (mag3110_i2c_addr | 0x01)    //  (BIT0 ==1 for reads)

extern const float PI;
extern const float oneEighty_div_pi;


extern unsigned short v;

unsigned short addr;

extern float fVal;

int offset_magX = 1682;
int offset_magY = -711;
int offset_magZ = 3307;

float slope_magX = 180.0/565.0;
float slope_magY = 180.0/570.0;
float slope_magZ = 180.0/(560.0/2);


//reads a 1 byte channel
void read_mag3110_chan1(unsigned short address, unsigned long *sensorData)
{

   unsigned short dat;

   addr = address;

   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(mag3110_W_ADDRESS, addr);
   *sensorData = (unsigned long)v;

}

//reads a 2 byte channel
void read_mag3110_chan2(unsigned short address, unsigned long *sensorData)
{

   unsigned short dat;

   addr = address;

   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(mag3110_W_ADDRESS, addr);
   *sensorData = (unsigned long)v<<8;
   //*sensorData = (unsigned long)v;

   addr = address + 1;  //low byte accelX
   dat = 0;
   v = 0;
   //v = LPS331_i2c_ReadReg(addr, &dat);
        v = i2c1_Get_Data(mag3110_W_ADDRESS, addr);
   *sensorData += ((unsigned long)v);


}



void init_mag3110()
{
   unsigned short address, dat;

// 1a) SET Output Data Rate to 1Hz
// 1b) SET SLEEP TO OFF   (REG: 0x20)
   addr = 0x11; //CTRL_REG2
   dat = 0xA0;     //set auto-mag sensor reset enabled, use raw readings (no offsets) rest defaults

   //LPS331_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mag3110_W_ADDRESS, addr, dat);

        
   addr = 0x10; //CTRL_REG1
   //dat = 0x0F<<3;     //set sleep off (to wakeup) & data-acq rate to 1.25Hz with 128over-sample
   dat = 1;     //set sleep off (to wakeup) & data-acq rate to 1.25Hz with 128over-sample

   //LPS331_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mag3110_W_ADDRESS, addr, dat);

   delay_ms(1);


}

char get_mag3110_who_am_i()
{
   unsigned short address, dat, val;
   //register of who_am_i: 0x75
   address = 0x07;
   dat = 0;

   //v = mpu9150_i2c_ReadReg(address, &dat);
	val = i2c1_Get_Data(mag3110_W_ADDRESS, address);


//should return dat = 0xC4
        return(val);
}


void get_mag3110_temp(int *temperature)
{
   unsigned int sens_data = 0;
   unsigned short addr, dat, val;
   //register of who_am_i: 0x75

   dat = 0;



    addr = 0x0F;  //temperature starting register addr
    
    read_mag3110_chan1(addr, &sens_data);  //returns single signed byte of temp in degC

    *temperature = (int)(sens_data);

}

int calc_magAngle2(int x, int y)
{
    if( (x <90) && (y <90) )
    {
        //quad 2 (270-360deg
        return(270 + x);
    }
    else if( (x <90) && (y >=90) )
    {
        //quad 3; (180-270degs)
        return(270 - x);

    }
    else if( (x >=90) && (y <90) )
    {
        //quad 1; (0-90degs)
        return(x - 90);

    }
    else if( (x >=90) && (y >=90) )
    {
        //quad 4; (90-180degs)
        return(y);

    }


}
float calc_magAngle(float x, float y)
{
    float rads = 0.0;
    float degs = 0.0;


    if(x != 0.0)
    {
        //C8/B8*(1+2/3*C8+2*4/(15)*C8^2+48/(3*5*7)*C8^3)  //note: c is y, and b is x
        //rads = y/x*(1.0+2.0/3.0*y+2.0*4.0/(15.0)*y^2+48.0/(3.0*5.0*7.0)*y^3);

        rads = atan2f(y, x);  //returns val from -pi to pi; takes 381tics => 9.5us

        //rads = Rajan_FastArcTan2(y, x);  //slower than lib routine; takes 706tics, versus 381tics for library "atan2f()"
    }
    else
    {
        if(y>0)
        {
            rads = PI;
        }
        else
        {
            rads = -PI;
        }
    }

    //degs = rads/(2.0*PI) * 360.0;
    degs = rads * 57.295779513082323;

    return(degs);
}

int get_mag3110_compass(short int *magX, short int *magY, short int *magZ)
{
   unsigned long sens_data = 0;
   float val;
   int heading;


    addr = 0x01;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magX = (short int)sens_data + offset_magX;  //sens_data/4096 => mBars

    addr = 0x03;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magY = (short int)sens_data + offset_magY;  //sens_data/4096 => mBars

    addr = 0x05;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magZ = (short int)sens_data + offset_magZ;  //sens_data/4096 => mBars


    val = (float)*magX * slope_magX;
    *magX = (short int)val;

    val = (float)*magY * slope_magY;
    *magY = (short int)val;

    val = (float)*magZ * slope_magZ;
    *magZ = (short int)val;

    //val = calc_magAngle((float)*magX,(float)*magY);
    heading = calc_magAngle2(*magX, *magY);
    
    return(heading);

}

int get_mag3110_compassXZ(short int *magX, short int *magY, short int *magZ)
{
   unsigned long sens_data = 0;
   float val;
   int heading;


    addr = 0x01;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magX = (short int)sens_data + offset_magX;  //sens_data/4096 => mBars

    addr = 0x03;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magY = (short int)sens_data + offset_magY;  //sens_data/4096 => mBars

    addr = 0x05;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magZ = (short int)sens_data + offset_magZ;  //sens_data/4096 => mBars


    val = (float)*magX * slope_magX;
    *magX = (short int)val;

    val = (float)*magY * slope_magY;
    *magY = (short int)val;

    val = (float)*magZ * slope_magZ;
    *magZ = (short int)val;

    //val = calc_magAngle((float)*magX,(float)*magY);
    heading = calc_magAngle2(*magZ, *magX);

    return(heading);

}

int get_mag3110_compassYZ(short int *magX, short int *magY, short int *magZ)
{
   unsigned long sens_data = 0;
   float val;
   int heading;


    addr = 0x01;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magX = (short int)sens_data + offset_magX;  //sens_data/4096 => mBars

    addr = 0x03;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magY = (short int)sens_data + offset_magY;  //sens_data/4096 => mBars

    addr = 0x05;  //pressure starting register address
    read_mag3110_chan2(addr, &sens_data);
    *magZ = (short int)sens_data + offset_magZ;  //sens_data/4096 => mBars


    val = (float)*magX * slope_magX;
    *magX = (short int)val;

    val = (float)*magY * slope_magY;
    *magY = (short int)val;

    val = (float)*magZ * slope_magZ;
    *magZ = (short int)val;

    //val = calc_magAngle((float)*magX,(float)*magY);
    heading = calc_magAngle2(*magY, *magZ);

    return(heading);

}