/* 
 * File:   main_5_9_14.c
 * Author: f
 *
 * Created on May 9, 2014, 10:26 AM
 */
#include <p32xxxx.h>
#include <plib.h>
#include <math.h>

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "delay_time.h"
#include "uart.h"
#include "i2c.h"
#include "SPI.H"

#include "stringUtils.h"
#include "adc.h"

#include "mpu9150_driver.h"
#include "LPS331_Pressure.h"
#include "mag3110_compass.h"

#include "setup.h"

#include "dataLog.h"


//slope intercept for weigher routing
float weigher_slope = 1.55161745051624e-5;
float weigher_intercept = -0.00001700413;


extern int loop_cnt;

extern duty_cycle1;
extern duty_cycle2;
//155 tics/msec or 15500tics/0.1secs on TMR1
extern const long tics_per_ms;
long tics_per_loop = 15500;  //default 10Hz update rate

//going to have an issue with mux on mcp3553 at 10Hz, so have to go slower or just get one channel per loop
//so need to toggle between precision ADC channels every time through loop

//vars for ISR for TMR4 (freq capture of sw1 & sw2)
volatile unsigned int val = 0;

volatile unsigned int an_selA_val = 0xFFFFFFFF;  //analog select register A
volatile unsigned int an_selB_val = 0xFFFFFFFF;  //analog select register B


volatile unsigned int last_sw1_val = 0;  //flag that let's us know our last transition
volatile unsigned int last_sw2_val = 0;


volatile unsigned int sw1_tic_cnt = 0;   //keeps running count of how many tics we've been high (need at least 8 for transition)
volatile unsigned int sw2_tic_cnt = 0;

volatile unsigned long total_tic_cnt1 = 0;
volatile unsigned long total_tic_cnt2 = 0;

volatile unsigned int sw1_period = 0;
volatile unsigned int sw2_period = 0;

volatile unsigned int period1 = 0;  //not a true period, more of a count
volatile unsigned int period2 = 0;

float freq1 = 0.0;
float freq2 = 0.0;

volatile unsigned int tics_without_toggle = 0;  //keep track of how many tics we have without a toggle
                                        //so we can call a zero freq.

volatile int semaphore = 0;  //used to sync ISR and main() so we don't access vals being modified by ISR
//end ISR vars


void run_test();

void main()
{
  char ok = 0;
  float val, num_us;
  unsigned int num_ms;
  short int X_AXI, Y_AXI, Z_AXI;
  long numTics = 0, ch0_data[14], ch0_data_sum, lt;
  int pressure_mBar, heading;
  int temp_degC,temp_degC1,temp_degC2, elev_ft, badReadCnt;
  short int *magX, *magY, *magZ;
  unsigned int notDataReady;
  int chan, chCnt=0;
  long tic_cnt = 0, i=0;

  int switchNum =1;
  unsigned char swVal;
  int reset=0;

   unsigned long pb_clk = GetPeripheralClock();

   //vars for communicating with ISR TMR4 poller for sw1, sw2 freq capture
   volatile unsigned long temp1 = 0;
   volatile unsigned long temp2 = 0;

    setup();

    initLogger();  //init on-board sensors, like compass, mpu, baro_press


//cpu module test for harvester project
//    run_test();


    

    //do
    //{
        sprintf((char*)pTxBuf, "Startup\r\n");
        SendDataBufferUART1(pTxBuf, strlen(pTxBuf));

    //    delay_ms(200);
    //}while(1);

    //test pwm lines; see void setup()  in setup.h   
    

/*
    do  //test switches
    {
        switchNum = 1;
        swVal = readSwitch(switchNum);
        switchNum = 2;
        swVal = readSwitch(switchNum);
    }while(1);
*/

    //RESET TMR1
    TMR1 = 0;

    //60k => delay 1 min
    //for(i=0;i<600;i++)  //DELAY 1000x/sec; TIMED & ACCURATE

    //delay 1 secs (1000 x 1ms in loop)
    for(i=0;i<10;i++)  //DELAY 1000x/sec; TIMED & ACCURATE
    {
        TMR1 = 0;
        do
        {
            tic_cnt = TMR1;

        //}while(TMR1 < tics_per_ms);   //6.45us/tic ==> 155tics/ms
        }while(TMR1 < tics_per_loop);   //0.1 Hz update
        
    }


    TMR1 = 0;

    do
    {
        //communicating with ISR TMR4 poller for sw1, sw2 freq capture
        IEC0bits.T4IE = 0;  //disable TMR4 interrupt before accessing variable that the ISR modifies (unless it's only a single byte
            temp1 = sw1_period;
            temp2 = sw2_period;
        IEC0bits.T4IE = 1;  //enable TMR4 interrupt

        period1 = temp1;
        period2 = temp2;

        //get some data; //for pressure, heading & tilt; takes 899tics => 6ms
//        runDataLogIteration( (unsigned char *)pTxBuf);

        tic_cnt = TMR1;

        tic_cnt++;

        chCnt = strlen(pTxBuf);
        
        //if we have data, txd it
        if(chCnt > 0)
        {
            SendDataBufferUART1(pTxBuf, chCnt);

            //now that data has been sent; zero out TxBuf so we won't send it again
            *pTxBuf = 0;
            
        }

        tic_cnt = TMR1;
        
        //check for setup update from PC
        //and delay for 200ms (our desired update rate)
        do
        {
            //getLine_uart1 uses coreTimer and resets it to get its timeout timing
            ok = getLine_uart1((unsigned char *)&RxBUFFER[0], maxChars, 200);  //getline with timeout=200ms
            if(ok==TRUE)
            {
                //rxd a setup string; run setup routine
                if(RxBUFFER[0]=='$')
                {
                    //test only; here we read mcp3551
                    
                    ok = configDataLogger((unsigned char *)&RxBUFFER[0], tics_per_ms, &tics_per_loop);
run_weigher();

                }
                
                break;
            }

            //during typical op (1 padc, 1 angle, 2 switches)
            //time to get here is 37,641 tics = 7.5ms
            tic_cnt = TMR1;
            
            //readSwitch at high rate to get best capture rates
            //for switch toggle timing (period or 1/freq)
            readSwitch(1);
            readSwitch(2);

        }while(TMR1 < tics_per_loop);  //HOLD IN LOOP TILL OUR LOOP EXIT TIME IS DONE

        //Reset TMR1
        TMR1 = 0;


    }while(1);

    
}


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // STEP 3. configure the Timer 4 interrupt handler

void __ISR(_TIMER_4_VECTOR, ipl2) Timer4Handler(void)
{
    tics_without_toggle++;  //keep track of # of tics without a toggle so we can call a zero freq
    if(tics_without_toggle > 2000)  //this should be about 2 secs or so
    {
        sw1_period = total_tic_cnt1;

        if(tics_without_toggle > 30000)
        {
            tics_without_toggle = 30000;  //keep from rolling over
        }
    }
    semaphore = 1;

    // clear the interrupt flag for Timer_1
    mT4ClearIntFlag();

    // .. things to do
    total_tic_cnt1++;
    total_tic_cnt2++;

    //val = PORTReadBits(IOPORT_B, BIT_1 | BIT_3);

    
    val = (~PORTA & BIT_0);


    if(val > 0)
    {

        if(last_sw1_val == 0)
        {
            sw1_tic_cnt++;
            if(sw1_tic_cnt >= 10)
            {
                sw1_period = total_tic_cnt1;
                total_tic_cnt1 = 0;
                last_sw1_val = 1; //so we don't double count
                
                tics_without_toggle = 0;
            }

        }

    }
    else
    {
        last_sw1_val = 0;
        sw1_tic_cnt = 0;
    }


    //analyze sw2
    val = ~(PORTA & BIT_1);

    if(val > 0)
    {

        if(last_sw2_val == 0)
        {
            sw2_tic_cnt++;
            if(sw2_tic_cnt >= 10)
            {
                sw2_period = total_tic_cnt2;
                total_tic_cnt2 = 0;
                last_sw2_val = 1; //so we don't double count
            }

        }

    }
    else
    {
        last_sw2_val = 0;
        sw2_tic_cnt = 0;
    }


    semaphore = 0;
}

void run_weigher()
{
    unsigned int spiPort = 1;
    unsigned int address = 2;  // 1 or 2 for module in slot 1; 6 or 6 for module in slot 2
    unsigned int stage;
    long adcVal;
    float volts;

    char* sData = 0;
    int id = 0;
    int i = 0;

    
    //run pwm till limit switches fire (or times out)
    duty_cycle1 = 100 - duty_cycle1;
    duty_cycle2 = 100 - duty_cycle2;
    
    turn_on_solenoid2();
    turn_off_solenoid34();

    run_PROGRAM2(sData, id);
    //delay 100 ms to ensure flow has stopped before turning off solenoid2

    turn_off_solenoid2();
    
    delay_ms(100);
    turn_on_solenoid34();

        stage = 0;  //2 part call, stage 0 to kick it off and stage 1 to get reading; must have 100ms between calls
        BOOL mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

        delay_ms(200);
        mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

    //LITTLE EXTRA INSURANCE; NOT GETTING SET IN ROUTING ALL THE TIME
    mPORTBSetBits(BIT_15);

    //get data  (24 readings)
    for(i=0;i<loop_cnt;i++)
    {
        stage = 0;  //2 part call, stage 0 to kick it off and stage 1 to get reading; must have 100ms between calls
        mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

        delay_ms(100);
        mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

        if(!mcp_ok)
        {
            delay_ms(50);
            mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);
        }

        //LITTLE EXTRA INSURANCE; NOT GETTING SET IN ROUTING ALL THE TIME
        mPORTBSetBits(BIT_15);


        volts = (float)(0x0FFFFFFF & adcVal);

        volts = volts* weigher_slope + weigher_intercept;


        sprintf((char*)pTxBuf, "data%d: %f\r\n",i,volts);
        SendDataBufferUART1(pTxBuf, strlen(pTxBuf));
    
    }


    turn_off_solenoid34();

}

//test for harvester project 2015
void run_test()
{
    int reset=0;
    
//note: cpu must be running (not paused in debug mode) for pwm to work
    //initPWM_Output(int pwmNum, int pwmFreq);
    reset = initPWM_Output(1, 100);  //get reset value (timer/clk max val)
    reset = initPWM_Output(2, 100);  //get reset value (timer/clk max val)
    reset = initPWM_Output(3, 100);  //get reset value (timer/clk max val)
    reset = initPWM_Output(4, 100);  //get reset value (timer/clk max val)
    //setPWM_Output(int pwmNum, int reset, int duty)
    //setPWM_Output(1, reset, 10);  //pass reset (timer/clk max val)

    setPWM_SolidStateRelay_Output(1, reset, 30);
    setPWM_SolidStateRelay_Output(2, reset, 50);
    setPWM_SolidStateRelay_Output(3, reset, 70);
    setPWM_SolidStateRelay_Output(4, reset, 90);

    //setPWM_SolidStateRelay_Output(1, reset, 0);
    //setPWM_SolidStateRelay_Output(1, reset, 10);
    //setPWM_SolidStateRelay_Output(1, reset, 20);
    //setPWM_SolidStateRelay_Output(1, reset, 50);
    //setPWM_SolidStateRelay_Output(1, reset, 60);
    //setPWM_SolidStateRelay_Output(1, reset, 70);
    //setPWM_SolidStateRelay_Output(1, reset, 75);
    //setPWM_SolidStateRelay_Output(1, reset, 80);
    //setPWM_SolidStateRelay_Output(1, reset, 90);
    //setPWM_SolidStateRelay_Output(1, reset, 95);
    //setPWM_SolidStateRelay_Output(1, reset, 100);

    //loop required to test pwm; comment it out to go to next test
    do
    {
        delay_ms(1);
    }while(1);

//test spi1 mcp3553

    unsigned int spiPort = 2;
    unsigned int address = 2;  // 1 or 2 for module in slot 1; 6 or 6 for module in slot 2
    unsigned int stage;
    long adcVal;

    stage = 0;  //2 part call, stage 0 to kick it off and stage 1 to get reading; must have 100ms between calls
    BOOL mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

    delay_ms(200);
    mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

    
    stage = 0;  //2 part call, stage 0 to kick it off and stage 1 to get reading; must have 100ms between calls
    mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);

    delay_ms(200);
    mcp_ok = run_mcp3553(spiPort, address, &stage, &adcVal);


//test limit_switches

//takes at least 2 high readings in a row to make a transition
    int switchNum = 1;
    unsigned char sw1_val = readSwitch(switchNum);
    sw1_val = readSwitch(switchNum);

    sw1_val = readSwitch(switchNum);
    sw1_val = readSwitch(switchNum);

    sw1_val = readSwitch(switchNum);
    sw1_val = readSwitch(switchNum);

    sw1_val = readSwitch(switchNum);
    sw1_val = readSwitch(switchNum);


    switchNum = 2;
    unsigned char sw2_val = readSwitch(switchNum);
    sw2_val = readSwitch(switchNum);

    sw2_val = readSwitch(switchNum);
    sw2_val = readSwitch(switchNum);

    sw2_val = readSwitch(switchNum);
    sw2_val = readSwitch(switchNum);

    sw2_val = readSwitch(switchNum);
    sw2_val = readSwitch(switchNum);

}