//mpu9150
//#include <math.h>

#include "i2c.h"
#include "math_utils.h"


#define mpu9150_i2c_addr 0xD0      //address: b110100xx  (2nd bit = 0 with ADO = gnd; 2nd bit = 1 with AD0=VCC)
                                   //1st bit is R/W with R==1 and W==0
                                   
#define mpu9150_W_ADDRESS   mpu9150_i2c_addr             //  ( BIT0 ==0 for writes)
#define mpu9150_R_ADDRESS   (mpu9150_i2c_addr | 0x01)    //  (BIT0 ==1 for reads)

const float PI = 3.141592653589793;
const float oneEighty_div_pi =  57.295779513082323;


void init_mpu9150()
{
    unsigned short address, dat;
   
// 1) SET FSYNC OFF AND BANDWIDTH OF ALL SENSORS to lowest possible: (REG: CONFIG 0x1A)
   address = 0x1A;
   dat = 6;     //set config to leave fsync off (0) and lowest bandwidth on sensors (0x06); highest BW set to 0x00

   //mpu9150_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mpu9150_W_ADDRESS, address, dat);


// 2) SET FIFO OFF  (REG: 0x6A)
   address = 0x6A;
   dat = 0;     //set USER CONTROL to SET FIFO off  (i.e.=0)

   //mpu9150_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mpu9150_W_ADDRESS, address, dat);


 // 3) SET Sample Rate Divider (REG: 0x19)  to 1
   address = 0x19;
   dat = 0;

   //mpu9150_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mpu9150_W_ADDRESS, address, dat);


 // 4) SET SCALE ON GYRO to lowest setting 250dps
   address = 0x1B;
   dat = 0;
   
   //mpu9150_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mpu9150_W_ADDRESS, address, dat);

// 5) SET SCALE ON ACCEL to lowest setting  +/-2g
   address = 0x1C;
   dat = 0;  //LOWEST SETTING +/-2g
   dat = 0x08;  //highest SETTING +/-4g
   //dat = 0x10;  //highest SETTING +/-8g
   dat = 0x18;  //highest SETTING +/-16g  (signed answer; +/- 1g just occupies less of 0-2^15 & 2^16-2^15+1

   
   //mpu9150_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mpu9150_W_ADDRESS, address, dat);



// 6a) SET CLOCK SOURCE   (REG: 0x6B, PWR MANAGEMENT 1)
// 6b) SET SLEEP TO OFF   (REG: 0x6B)
   address = 0x6B;
   dat = 1;     //set sleep off (to wakeup) & set internal clock to gyro X axis (suggested by data-sheet)

   //mpu9150_i2c_WriteReg(address, dat);
	i2c1_Send_DataReg(mpu9150_W_ADDRESS, address, dat);
   
   
   delay_ms(1);
   
}

char get_mpu9150_who_am_i()
{
   unsigned short address, dat, val;
   //register of who_am_i: 0x75
   address = 0x75;
   dat = 0;
   
   //v = mpu9150_i2c_ReadReg(address, &dat);
	val = i2c1_Get_Data(mpu9150_W_ADDRESS, address);

//should return dat = 0x68 = 104d
}

float convert_rads2degs(float rads)
{
    float degs;

    //degs = rads/(2.0*pi) * 360.0;
    degs = rads * oneEighty_div_pi;

    return(degs);
}


float calc_angle(float x, float y)
{
    float rads = 0.0;
    float degs = 0.0;


    if(x != 0.0)
    {
        //C8/B8*(1+2/3*C8+2*4/(15)*C8^2+48/(3*5*7)*C8^3)  //note: c is y, and b is x
        //rads = y/x*(1.0+2.0/3.0*y+2.0*4.0/(15.0)*y^2+48.0/(3.0*5.0*7.0)*y^3);

        rads = atan2f(y, x);  //returns val from -pi to pi; takes 381tics => 9.5us

        //rads = Rajan_FastArcTan2(y, x);  //slower than lib routine; takes 706tics, versus 381tics for library "atan2f()"
    }
    else
    {
        if(y>0)
        {
            rads = PI;
        }
        else
        {
            rads = -PI;
        }
    }
    
    //degs = rads/(2.0*PI) * 360.0;
    degs = rads * 57.295779513082323;

    return(degs);
}

void read_mpu9150_chan(unsigned short address, unsigned int *sensorData)
{  
   unsigned short addr = address;
   unsigned short dat, val;
   
   dat = 0;
   val = 0;
   //val = mpu9150_i2c_ReadReg(addr, &dat);
	val = i2c1_Get_Data(mpu9150_W_ADDRESS, addr);
   *sensorData = val<<8;

   addr++;  //low byte accelX
   dat = 0;
   val = 0;
   //val = mpu9150_i2c_ReadReg(addr, &dat);
	val = i2c1_Get_Data(mpu9150_W_ADDRESS, addr);
   *sensorData += val;


}

//short int is 16 bits, int is 32bits
void get_mpu9150_accel(short int *accelX, short  int *accelY, short  int *accelZ)
{
   unsigned int sens_data = 0;
   unsigned short address;
   float angle = 0.0;

   
    address = 0x3B;  //high byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *accelX = sens_data<<8;

    address = 0x3C;  //high byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *accelX += sens_data;


    address = 0x3D;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *accelY = sens_data<<8;

    address = 0x3E;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *accelY += sens_data;


    angle = calc_angle((float)*accelY, (float)*accelX);

    
    address = 0x3F;  //high byte accelZ
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *accelZ = sens_data<<8;

    address = 0x40;  //high byte accelZ
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *accelZ += sens_data;


}

//get tilt estimate
//64162tics (1.6ms) to complete this call  (vast majority of this time is in the I2C calls...)
float get_mpu9150_Tilt_XY()
{
   unsigned int sens_data = 0;
   unsigned short address;
   short int accelX, accelY, accelZ;
   float angle = 0.0;
   long numTics = 0;

   WriteCoreTimer(0);		// reset clock

   address = 0x3B;  //high byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelX = sens_data<<8;

    address = 0x3C;  //low byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelX += sens_data;


    address = 0x3D;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelY = sens_data<<8;

    address = 0x3E;  //low byte accelY
    //read_mpu9150_chan(address, &sens_data);
        //this takes 15943tics (0.4ms) [doesn't seem to matter what the clock rate is on I2C...
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelY += sens_data;


   //takes 9.5usecs to calc this
    angle = calc_angle((float)accelY, (float)accelX) + 180.0;

   numTics = ReadCoreTimer();

    return(angle);
}

float get_mpu9150_Tilt_YZ()
{
   unsigned int sens_data = 0;
   unsigned short address;
   short int accelZ, accelY;
   float angle = 0.0;
   long numTics = 0;

   WriteCoreTimer(0);		// reset clock

   address = 0x3F;  //high byte accelZ
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelZ = sens_data<<8;

    address = 0x40;  //low byte accelZ
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelZ += sens_data;


    address = 0x3D;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelY = sens_data<<8;

    address = 0x3E;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
        //this takes 15943tics (0.4ms) [doesn't seem to matter what the clock rate is on I2C...
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelY += sens_data;


   //takes 9.5usecs to calc this
    angle = calc_angle((float)accelY, (float)accelZ) + 180.0;

   numTics = ReadCoreTimer();

    return(angle);
}

float get_mpu9150_Tilt_XZ()
{
   unsigned int sens_data = 0;
   unsigned short address;
   short int accelX, accelZ;
   float angle = 0.0;
   long numTics = 0;

   WriteCoreTimer(0);		// reset clock

   address = 0x3B;  //high byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelX = sens_data<<8;

    address = 0x3C;  //low byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelX += sens_data;


    address = 0x3F;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelZ = sens_data<<8;

    address = 0x40;  //low byte accelY
    //read_mpu9150_chan(address, &sens_data);
        //this takes 15943tics (0.4ms) [doesn't seem to matter what the clock rate is on I2C...
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelZ += sens_data;


   //takes 9.5usecs to calc this
    angle = calc_angle((float)accelZ, (float)accelX) + 180.0;

   numTics = ReadCoreTimer();

    return(angle);
}

float get_mpu9150_angle()
{
   unsigned int sens_data = 0;
   unsigned short address;
   short int accelX, accelY;
   float angle = 0.0;
   long numTics = 0;

   WriteCoreTimer(0);		// reset clock

   address = 0x3B;  //high byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelX = sens_data<<8;

    address = 0x3C;  //low byte accelX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelX += sens_data;


    address = 0x3D;  //high byte accelY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelY = sens_data<<8;

    address = 0x3E;  //low byte accelY
    //read_mpu9150_chan(address, &sens_data);
        //this takes 15943tics (0.4ms) [doesn't seem to matter what the clock rate is on I2C...
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    accelY += sens_data;


   //takes 9.5usecs to calc this
    angle = calc_angle((float)accelY, (float)accelX) + 180.0;

   numTics = ReadCoreTimer();

    return(angle);
}

void get_mpu9150_gyro(unsigned int *gyroX, unsigned int *gyroY, unsigned int *gyroZ)
{
   unsigned int sens_data = 0;
   unsigned short address;

    address = 0x43;  //high byte gyroX
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *gyroX = sens_data;

    address = 0x45;  //high byte gyroY
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *gyroY = sens_data;

    address = 0x47;  //high byte gyroZ
    //read_mpu9150_chan(address, &sens_data);
	sens_data = i2c1_Get_Data(mpu9150_W_ADDRESS, address);
    *gyroZ = sens_data;



}

void get_mpu9150_temperature(unsigned int *temperature)
{
   unsigned int sens_data = 0;
   unsigned short address;

    address = 0x41;  //high byte temperature
    read_mpu9150_chan(address, &sens_data);
    *temperature = sens_data;


}