//Setup digital and analog channels
#include <plib.h>


//#include "adxl345_driver.h"
#include "mpu9150_driver.h"
#include "LPS331_Pressure.h"
#include "mag3110_compass.h"
#include "mcp3553_driver.h"

//#include "spi.h"

#define SYS_FREQ             (40000000L)
#define PB_DIV                 1
#define PRESCALE               64
#define TOGGLES_PER_SEC        1000
#define TMR_TICK               (SYS_FREQ/PB_DIV/PRESCALE/TOGGLES_PER_SEC)

#define PRESCALE4               1
#define TOGGLES_PER_SEC4        1500  //5ms tic period (

//155 tics/msec; timed and accurate
const long tics_per_ms = 155;

extern volatile unsigned int an_selA_val;   //analog select register A
extern volatile unsigned int an_selB_val;   //analog select register B

//  ******************* Stand-alone Logging SETUP  *****************************
//#define dBufSize 1000
//UINT8 DataBUFFER[dBufSize];
//  char* pDataBuf = (char*) &DataBUFFER[0];






void setup()
{
  char ok = 0;
  float val, num_us;
  unsigned int num_ms;
  short int X_AXI, Y_AXI, Z_AXI, i;
  long numTics = 0, ch0_data[14], ch0_data_sum, lt, numTics2=0;
  int pressure_mBar, heading;
  int temp_degC,temp_degC1,temp_degC2, elev_ft, badReadCnt;
  short int *magX, *magY, *magZ;
  unsigned int notDataReady;
  int chan;

  //just hard coded this into uart_init, so just set to zero for now
  int use_cs0_rb15=0; int use_cs2_rb7=0; int use_cs3_rb6=0; int use_cs1_rb12=0;
  
  unsigned long T4_TICK = SYS_FREQ/PB_DIV/PRESCALE4/TOGGLES_PER_SEC4;

    //SYSTEMConfig(SYS_FREQ, 0);


	//init main processor 
	pbClk = SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);


	mPMPClose(); //need to make sure PMP is off or you won't be able to use shared DIO on the PMP pins, like A3	

        //init TMR4 for interrupt ISR with tmr rate of 5ms
       OpenTimer4(T4_ON | T4_SOURCE_INT | T4_PS_1_1, T4_TICK);

        // set up the timer interrupt with a priority of 2 (7 is highest priority, 0 is least)
        ConfigIntTimer4(T4_INT_ON | T4_INT_PRIOR_2);

        numTics = TMR4;
        delay_ms(10);
        numTics2 = TMR4;

        //not here!!! IN initLogger()  [LINE 1310 in DATALOG.C]
        // enable multi-vector interrupts
        //INTEnableSystemMultiVectoredInt();
        //IEC0bits.T4IE = 0;  //disable TMR4 interrupt before accessing variable that the ISR modifies (unless it's only a single byte
        //temp1 = sw1_period;
        //temp2 = sw2_period;
        //IEC0bits.T4IE = 1;  //enable TMR4 interrupt


//controls hyd valve so put it first so it'll turn-off asap
        //mPORTBSetBits(BIT_6);  //neg logic
        //mPORTBSetBits(BIT_7);  //neg logic

//old version
//        mPORTBClearBits(BIT_6);  //neg logic
//        mPORTBClearBits(BIT_7);  //neg logic
//	PORTSetPinsDigitalOut(IOPORT_B, BIT_6);  //pwm1; no analog, just digital
//	PORTSetPinsDigitalOut(IOPORT_B, BIT_7);  //pwm2; no analog, just digital

//2015 version  (NOTE: CALL SETUP BEFORE UARTx_INIT()
        ANSELA = 0xffffffff;  //default value; really shouldn't have to do this
        ANSELB = 0xffffffff;
        an_selA_val = ANSELA;
        an_selB_val = ANSELB;

        
	ANSELB = an_selB_val & (~BIT_1) & (~BIT_2);	//must de-select all analog select bits for any digital I/O or digital function
        an_selB_val = ANSELB;



        mPORTBClearBits(BIT_1);  //neg logic
        mPORTBClearBits(BIT_2);  //neg logic
        mPORTBClearBits(BIT_4);  //neg logic
        mPORTAClearBits(BIT_4);  //neg logic
	PORTSetPinsDigitalOut(IOPORT_B, BIT_1);  //pwm1; no analog, just digital
	PORTSetPinsDigitalOut(IOPORT_B, BIT_2);  //pwm2; no analog, just digital
	PORTSetPinsDigitalOut(IOPORT_B, BIT_4);  //pwm3; no analog, just digital
	PORTSetPinsDigitalOut(IOPORT_A, BIT_4);  //pwm4; no analog, just digital


     //good test but best not to leave this active in real system where it'll fire solenoids
        mPORTBClearBits(BIT_1);
        mPORTBSetBits(BIT_1);
        mPORTBClearBits(BIT_1);
        mPORTBSetBits(BIT_1);
        mPORTBClearBits(BIT_1);
        //mPORTBSetBits(BIT_1);

        mPORTBClearBits(BIT_2);
        mPORTBSetBits(BIT_2);
        mPORTBClearBits(BIT_2);
        mPORTBSetBits(BIT_2);
        mPORTBClearBits(BIT_2);
        //mPORTBSetBits(BIT_2);

        mPORTBClearBits(BIT_4);
        mPORTBSetBits(BIT_4);
        mPORTBClearBits(BIT_4);
        mPORTBSetBits(BIT_4);
        mPORTBClearBits(BIT_4);
        //mPORTBSetBits(BIT_4);

        mPORTAClearBits(BIT_4);
        mPORTASetBits(BIT_4);
        mPORTAClearBits(BIT_4);
        mPORTASetBits(BIT_4);
        mPORTAClearBits(BIT_4);
        //mPORTASetBits(BIT_4);

	//uart pin mapping done in uart1Init()
	//U1RXRbits.U1RXR = 2; //SET RPA4 to RX
	//RPB4Rbits.RPB4R = 1; //SET RPB4 to TX
	uart1Init();	//note: assumes use of single sharted TX buffer so uart1 can't run at same time as uart2 without code change to add seperate buffer rather than a shared 1
//	uart2Init();	//note: assumes use of single sharted TX buffer so uart1 can't run at same time as uart2 without code change to add seperate buffer rather than a shared 1


        //setup pins for SPI (specifically CS3 that MCP3551/3553 is using
        //RB6 is CS3 9no analog
//	PORTSetPinsDigitalOut(IOPORT_B, BIT_6);  //CS3; no analog, just digital
//	PORTSetPinsDigitalOut(IOPORT_B, BIT_7);  //CS2; no analog, just digital


        //sck1 RB14 (an10)
        //sdo1 RB13 (an11)
        //sdi1 RB11 (no analog)
        



	//setup pins for i2c1
//moved to i2c_init()
//	PORTSetPinsDigitalOut(IOPORT_B, BIT_9);  //I2C1, SDA; no analog, just digital
//	PORTSetPinsDigitalOut(IOPORT_B, BIT_8);	 //I2C1, SCL; no analog, just digital

        

        //setup timer 2 (16bit mode with pre-scaler to implement ms timer
        //OpenTimer1(T1_ON | T1_SOURCE_INT | T1_PS_1_256, T1_TICK);
        //OpenTimer23(T23_ON | T23_PS_1_64, 65000);
        
        T1CON = 0x8030;  //SET TIMER 2-3 AS 32 BIT TIMER WITH 128:1 Prescaler
        PR1 = 0xFFFF;


        
        //OpenTimer45(T45_ON | T45_PS_1_256, 0xFFFF);

        //no mapping for i2c1, dedicated pins

        //move from here into main so we can free up spi1 & cs lines for other uses
//        spi1_init();
        //void spi1_init(int use_cs0_rb15, int use_cs2_rb7, int use_cs3_rb6, int use_cs1_rb12)
        spi1_init(use_cs0_rb15, use_cs2_rb7, use_cs3_rb6, use_cs1_rb12);



//	init_i2c(1);


        return;

//mcp3553 code works
    stop_mcp3553(1);

    while(1)
    {
        //throw away data
        ch0_data_sum = 0;
        badReadCnt = 0;

reset_Time();

        for(i=0;i<9;i++)
        {
            start_mcp3553(1);

            delay_ms(1);
            notDataReady = 1;
            do
            {
                notDataReady = spi1_check_mcp3553();
                delay_ms(10); //need a small delay here
            }while(notDataReady>0);

            //take SPI readings and process data
            lt = spi1_process_mcp3553();

            if(lt > 0)
            {
                ch0_data[i] = lt;
                if(i>0)  //throw out 1st 4
                {
                    ch0_data_sum += lt;
                }
            }
            else
            {
                i--;
                badReadCnt++;
            }

            //continuous mode doesn't seem to work... so do this here; cost is we take a few extra readings in each series
            stop_mcp3553(1);

            delay_ms(10);
        }


        ch0_data_sum = ch0_data_sum>>3;
num_ms = get_Time_ms();

        ch0_data_sum = 0;



    }


        return;

        





        
	init_mpu9150();
 	ok = get_mpu9150_who_am_i();
	
//do
//{
    val = get_mpu9150_angle();  //takes 64,100 tics => 1.6ms


    get_mpu9150_accel(&X_AXI,&Y_AXI,&Z_AXI);
    
//}while(1);

    //setup pressure sensor
    init_LPS331();

    ok = get_LPS331_who_am_i();
    //should return okt = 0xBB = 187d

    //get pressure (mBar)
    get_LPS331_pressure(&pressure_mBar);

    //temp doesn't work...
    //get_LPS331_temperature(&temp_degC);
            
    //convert to sea Level (mBar)
    elev_ft = 3010;
    temp_degC = 20;
    pressure_mBar = convert_pres_to_seaLevel((int) pressure_mBar, elev_ft, temp_degC);


    //compass mag3110
    init_mag3110();
    ok = get_mag3110_who_am_i();

    //TEMP DOESN'T SEEM TO WORK...
    get_mag3110_temp(&temp_degC);

 //do
 //{
    get_mag3110_temp(&temp_degC);
    heading = get_mag3110_compass(&magX, &magY, &magZ);
    ok = get_mag3110_who_am_i();
 //}while(1);
    

    return;
    







    
//test thermo-click (max31855)  doesn't work; noise?  seems like it's reading ok (especially ref temp &
    //when thermocouple is unplugged; gets correct values...
/*    //works but actual hardware not so good; 
    do
    {
        chan = 7;
        temp_degC1 = MAX31855_Read(chan);

        delay_ms(200);

        //note: update rate is slower than 70ms!!!
        chan = 7;
        temp_degC2 = MAX31855_Read(chan);


        //note: update rate is slower than 70ms!!! so must delay!!!
        delay_ms(200);

    }while(0);
*/


//test ADC (precision 22bit)  //works

    stop_mcp3553(7);

    while(1)
    {
        //throw away data
        ch0_data_sum = 0;
        badReadCnt = 0;

reset_Time();

        for(i=0;i<9;i++)
        {
            start_mcp3553(7);

            delay_ms(1);
            notDataReady = 1;
            do
            {
                notDataReady = spi1_check_mcp3553();
                delay_ms(10); //need a small delay here
            }while(notDataReady>0);

            //take SPI readings and process data
            lt = spi1_process_mcp3553();

            if(lt > 0)
            {
                ch0_data[i] = lt;
                if(i>0)  //throw out 1st 4
                {
                    ch0_data_sum += lt;
                }
            }
            else
            {
                i--;
                badReadCnt++;
            }

            //continuous mode doesn't seem to work... so do this here; cost is we take a few extra readings in each series
            stop_mcp3553(7);

            delay_ms(10);
        }


        ch0_data_sum = ch0_data_sum>>3;
num_ms = get_Time_ms();

        ch0_data_sum = 0;


        
    }



 
/*

	initADC_autoSample();
	//temperature = readADC_ManualStart_AutoConvert(1);

        //this was already done earlier
	mPMPClose(); //need to make sure PMP is off or you won't be able to use shared DIO on the PMP pins, like A3	



//	ANSELB = (~BIT_0)| (~BIT_11)|(~BIT_12)|(~BIT_13)|(~BIT_14)  ;	//must de-select all analog select bits for any digital I/O or digital function 
	ANSELB = (~BIT_11)|(~BIT_12)|(~BIT_10)  ;	//must de-select all analog select bits for any digital I/O or digital function 

    PORTSetPinsDigitalOut(IOPORT_B, BIT_12);	//LE pin23; RB12/AN12
    PORTSetPinsDigitalOut(IOPORT_B, BIT_13);	//sdo pin24; RB13/AN11
    PORTSetPinsDigitalOut(IOPORT_B, BIT_14);	//sck pin25; RB14/AN10

    PORTSetPinsDigitalIn(IOPORT_B, BIT_11);		//lock detect, pin 22, RB11 (no analog), (note: 5V tolerant on input)

    //PORTSetPinsDigitalOut(IOPORT_A, BIT_0);		//led on microstick board


    PORTSetPinsDigitalOut(IOPORT_B, BIT_5);		//pin 14, RB5, switch 0 (no analog)
    PORTSetPinsDigitalOut(IOPORT_B, BIT_6);		//pin 15, RB6, switch 1 (no analog)
    PORTSetPinsDigitalOut(IOPORT_B, BIT_7);		//pin 16, RB7, Bale LED (no analog)


	int freq = 914;
	initFreqSynth(freq);
*/

}
